@@ -49,8 +49,7 @@ struct _GstLogoinsert
 {
   GstBaseTransform base_transform;
 
-  char *location;
-  GstBuffer *buffer;
+  gchar *location;
 
   GstVideoFormat format;
   int width;
@@ -59,7 +58,7 @@ struct _GstLogoinsert
   gchar *data;
   gsize size;
   CogFrame *overlay_frame;
-  CogFrame *argb_frame;
+  CogFrame *ayuv_frame;
   CogFrame *alpha_frame;
 
 };
@@ -70,28 +69,33 @@ struct _GstLogoinsertClass
 
 };
 
+
+/* GstLogoinsert signals and args */
+enum
+{
+  /* FILL ME */
+  LAST_SIGNAL
+};
+
 enum
 {
   ARG_0,
   ARG_LOCATION,
-  ARG_DATA
 };
 
 GType gst_logoinsert_get_type (void);
 
-GST_DEBUG_CATEGORY_STATIC (gst_logoinsert_debug_category);
-#define GST_CAT_DEFAULT gst_logoinsert_debug_category
+static void gst_logoinsert_base_init (gpointer g_class);
+static void gst_logoinsert_class_init (gpointer g_class, gpointer class_data);
+static void gst_logoinsert_init (GTypeInstance * instance, gpointer g_class);
 
 static void gst_logoinsert_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_logoinsert_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static void gst_logoinsert_dispose (GObject * object);
-static void gst_logoinsert_finalize (GObject * object);
 
-static void
-gst_logoinsert_set_location (GstLogoinsert * li, const char *location);
-static void gst_logoinsert_set_data (GstLogoinsert * li, GstBuffer * buffer);
+static void gst_logoinsert_set_location (GstLogoinsert * li,
+    const gchar * location);
 
 static gboolean gst_logoinsert_set_caps (GstBaseTransform * base_transform,
     GstCaps * incaps, GstCaps * outcaps);
@@ -112,64 +116,76 @@ static GstStaticPadTemplate gst_logoinsert_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("{I420,YUY2,UYVY,AYUV}"))
     );
 
-/* class initialization */
+GType
+gst_logoinsert_get_type (void)
+{
+  static GType compress_type = 0;
 
-#define DEBUG_INIT(bla) \
-  GST_DEBUG_CATEGORY_INIT (gst_logoinsert_debug_category, "logoinsert", 0, \
-      "debug category for logoinsert element");
+  if (!compress_type) {
+    static const GTypeInfo compress_info = {
+      sizeof (GstLogoinsertClass),
+      gst_logoinsert_base_init,
+      NULL,
+      gst_logoinsert_class_init,
+      NULL,
+      NULL,
+      sizeof (GstLogoinsert),
+      0,
+      gst_logoinsert_init,
+    };
+
+    compress_type = g_type_register_static (GST_TYPE_BASE_TRANSFORM,
+        "GstLogoinsert", &compress_info, 0);
+  }
+  return compress_type;
+}
 
-GST_BOILERPLATE_FULL (GstLogoinsert, gst_logoinsert, GstBaseTransform,
-    GST_TYPE_BASE_TRANSFORM, DEBUG_INIT);
 
 static void
 gst_logoinsert_base_init (gpointer g_class)
 {
-
+  static GstElementDetails compress_details =
+      GST_ELEMENT_DETAILS ("Video Filter Template",
+      "Filter/Effect/Video",
+      "Template for a video filter",
+      "David Schleef <ds@schleef.org>");
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_logoinsert_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_logoinsert_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_logoinsert_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_logoinsert_sink_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "Overlay image onto video", "Filter/Effect/Video",
-      "Overlay image onto video", "David Schleef <ds@schleef.org>");
+  gst_element_class_set_details (element_class, &compress_details);
 }
 
 static void
-gst_logoinsert_class_init (GstLogoinsertClass * klass)
+gst_logoinsert_class_init (gpointer g_class, gpointer class_data)
 {
   GObjectClass *gobject_class;
   GstBaseTransformClass *base_transform_class;
+  GstLogoinsertClass *filter_class;
 
-  gobject_class = G_OBJECT_CLASS (klass);
-  base_transform_class = GST_BASE_TRANSFORM_CLASS (klass);
+  gobject_class = G_OBJECT_CLASS (g_class);
+  base_transform_class = GST_BASE_TRANSFORM_CLASS (g_class);
+  filter_class = GST_LOGOINSERT_CLASS (g_class);
 
   gobject_class->set_property = gst_logoinsert_set_property;
   gobject_class->get_property = gst_logoinsert_get_property;
-  gobject_class->dispose = gst_logoinsert_dispose;
-  gobject_class->finalize = gst_logoinsert_finalize;
 
   g_object_class_install_property (gobject_class, ARG_LOCATION,
       g_param_spec_string ("location", "location",
-          "location of PNG file to overlay", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_DATA,
-      gst_param_spec_mini_object ("data", "data",
-          "Buffer containing PNG file to overlay", GST_TYPE_BUFFER,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "location of PNG file to overlay", "", G_PARAM_READWRITE));
 
   base_transform_class->set_caps = gst_logoinsert_set_caps;
   base_transform_class->transform_ip = gst_logoinsert_transform_ip;
 }
 
 static void
-gst_logoinsert_init (GstLogoinsert * logoinsert,
-    GstLogoinsertClass * logoinsert_class)
+gst_logoinsert_init (GTypeInstance * instance, gpointer g_class)
 {
 
   GST_DEBUG ("gst_logoinsert_init");
@@ -189,10 +205,6 @@ gst_logoinsert_set_property (GObject * object, guint prop_id,
     case ARG_LOCATION:
       gst_logoinsert_set_location (src, g_value_get_string (value));
       break;
-    case ARG_DATA:
-      gst_logoinsert_set_data (src,
-          (GstBuffer *) gst_value_get_mini_object (value));
-      break;
     default:
       break;
   }
@@ -211,53 +223,12 @@ gst_logoinsert_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_LOCATION:
       g_value_set_string (value, src->location);
       break;
-    case ARG_DATA:
-      gst_value_set_mini_object (value, (GstMiniObject *) src->buffer);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-void
-gst_logoinsert_dispose (GObject * object)
-{
-  g_return_if_fail (GST_IS_LOGOINSERT (object));
-
-  /* clean up as possible.  may be called multiple times */
-
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-}
-
-void
-gst_logoinsert_finalize (GObject * object)
-{
-  GstLogoinsert *logoinsert;
-
-  g_return_if_fail (GST_IS_LOGOINSERT (object));
-  logoinsert = GST_LOGOINSERT (object);
-
-  g_free (logoinsert->location);
-  if (logoinsert->buffer) {
-    gst_buffer_unref (logoinsert->buffer);
-  }
-  if (logoinsert->overlay_frame) {
-    cog_frame_unref (logoinsert->overlay_frame);
-    logoinsert->overlay_frame = NULL;
-  }
-  if (logoinsert->alpha_frame) {
-    cog_frame_unref (logoinsert->alpha_frame);
-    logoinsert->alpha_frame = NULL;
-  }
-  if (logoinsert->argb_frame) {
-    cog_frame_unref (logoinsert->argb_frame);
-    logoinsert->argb_frame = NULL;
-  }
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
 static gboolean
 gst_logoinsert_set_caps (GstBaseTransform * base_transform,
     GstCaps * incaps, GstCaps * outcaps)
@@ -281,25 +252,21 @@ gst_logoinsert_transform_ip (GstBaseTransform * base_transform, GstBuffer * buf)
   g_return_val_if_fail (GST_IS_LOGOINSERT (base_transform), GST_FLOW_ERROR);
   li = GST_LOGOINSERT (base_transform);
 
-  if (li->argb_frame == NULL)
-    return GST_FLOW_OK;
+  frame = gst_cog_buffer_wrap (buf, li->format, li->width, li->height);
 
-  frame = gst_cog_buffer_wrap (gst_buffer_ref (buf),
-      li->format, li->width, li->height);
+  if (li->ayuv_frame == NULL)
+    return GST_FLOW_OK;
 
   if (li->overlay_frame == NULL) {
     CogFrame *f;
 
-    f = cog_virt_frame_extract_alpha (cog_frame_ref (li->argb_frame));
-    f = cog_virt_frame_new_subsample (f, frame->format,
-        COG_CHROMA_SITE_MPEG2, 2);
+    f = cog_virt_frame_extract_alpha (cog_frame_ref (li->ayuv_frame));
+    f = cog_virt_frame_new_subsample (f, frame->format);
     li->alpha_frame = cog_frame_realize (f);
 
-    f = cog_virt_frame_new_unpack (cog_frame_ref (li->argb_frame));
-    f = cog_virt_frame_new_color_matrix_RGB_to_YCbCr (f, COG_COLOR_MATRIX_SDTV,
-        8);
-    f = cog_virt_frame_new_subsample (f, frame->format,
-        COG_CHROMA_SITE_MPEG2, 2);
+    f = cog_virt_frame_new_unpack (cog_frame_ref (li->ayuv_frame));
+    f = cog_virt_frame_new_color_matrix_RGB_to_YCbCr (f);
+    f = cog_virt_frame_new_subsample (f, frame->format);
     li->overlay_frame = cog_frame_realize (f);
   }
 
@@ -333,65 +300,41 @@ gst_logoinsert_transform_ip (GstBaseTransform * base_transform, GstBuffer * buf)
     }
   }
 
-  cog_frame_unref (frame);
 
   return GST_FLOW_OK;
 }
 
-static GstBuffer *
-get_buffer_from_file (const char *filename)
+static void
+gst_logoinsert_set_location (GstLogoinsert * li, const gchar * location)
 {
   gboolean ret;
-  GstBuffer *buffer;
-  gsize size;
-  gchar *data;
+  GError *error = NULL;
 
-  ret = g_file_get_contents (filename, &data, &size, NULL);
-  if (!ret)
-    return NULL;
-
-  buffer = gst_buffer_new ();
-  GST_BUFFER_DATA (buffer) = (guchar *) data;
-  GST_BUFFER_MALLOCDATA (buffer) = (guchar *) data;
-  GST_BUFFER_SIZE (buffer) = size;
-
-  return buffer;
-}
-
-static void
-gst_logoinsert_set_location (GstLogoinsert * li, const char *location)
-{
   g_free (li->location);
-  li->location = g_strdup (location);
-
-  gst_logoinsert_set_data (li, get_buffer_from_file (li->location));
-}
-
-static void
-gst_logoinsert_set_data (GstLogoinsert * li, GstBuffer * buffer)
-{
-  if (li->buffer)
-    gst_buffer_unref (li->buffer);
-
-  /* steals the reference */
-  li->buffer = buffer;
-
+  g_free (li->data);
   if (li->overlay_frame) {
     cog_frame_unref (li->overlay_frame);
     li->overlay_frame = NULL;
   }
+
+  li->location = g_strdup (location);
+
+  ret = g_file_get_contents (li->location, &li->data, &li->size, &error);
+  if (!ret) {
+    li->data = NULL;
+    li->size = 0;
+    return;
+  }
+
+  li->ayuv_frame = cog_frame_new_from_png (li->data, li->size);
+
   if (li->alpha_frame) {
     cog_frame_unref (li->alpha_frame);
     li->alpha_frame = NULL;
   }
-  if (li->argb_frame) {
-    cog_frame_unref (li->argb_frame);
-    li->argb_frame = NULL;
-  }
-
-  if (li->buffer) {
-    li->argb_frame = cog_frame_new_from_png (GST_BUFFER_DATA (li->buffer),
-        GST_BUFFER_SIZE (li->buffer));
+  if (li->overlay_frame) {
+    cog_frame_unref (li->overlay_frame);
+    li->overlay_frame = NULL;
   }
 }
 
@@ -408,7 +351,7 @@ struct png_data_struct
 static void
 read_data (png_structp png_ptr, png_bytep data, png_size_t length)
 {
-  struct png_data_struct *s = png_get_io_ptr (png_ptr);
+  struct png_data_struct *s = png_ptr->io_ptr;
 
   memcpy (data, s->data + s->offset, length);
   s->offset += length;
@@ -423,6 +366,7 @@ cog_frame_new_from_png (void *data, int size)
   png_bytep *rows;
   CogFrame *frame;
   guchar *frame_data;
+  int rowbytes;
   int j;
   int width, height;
   int color_type;
@@ -451,9 +395,9 @@ cog_frame_new_from_png (void *data, int size)
   }
 
   frame_data = g_malloc (width * height * 4);
-  frame = cog_frame_new_from_data_ARGB (frame_data, width, height);
-  frame->regions[0] = frame_data;
+  frame = cog_frame_new_from_data_AYUV (frame_data, width, height);
 
+  rowbytes = png_get_rowbytes (png_ptr, info_ptr);
   rows = (png_bytep *) g_malloc (sizeof (png_bytep) * height);
 
   for (j = 0; j < height; j++) {
@@ -462,7 +406,7 @@ cog_frame_new_from_png (void *data, int size)
   png_read_image (png_ptr, rows);
   g_free (rows);
 
-  png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp) NULL);
+  png_destroy_read_struct (&png_ptr, &info_ptr, png_infopp_NULL);
 
   return frame;
 }
