@@ -20,9 +20,9 @@
 
 /**
  * SECTION:element-faac
- * @see_also: faad
+ * @seealso: faad
  *
- * faac encodes raw audio to AAC (MPEG-4 part 3) streams.
+ * faac encodes raw audio to AAC (MPEG-4 part 10) streams.
  *
  * The #GstFaac:outputformat property determines whether or not the
  * AAC data needs additional framing provided by a container
@@ -31,14 +31,16 @@
  * framing and needs no container.
  *
  * The #GstFaac:profile property determines the AAC profile, where the default
- * LC (Low Complexity) profile is most widely used, supported and suitable for
- * general use. The other profiles are very rarely used and often not supported.
+ * Main profile is fine for most players and settings,
+ * but in some cases (e.g. hardware platforms)
+ * a more restricted profile/level may be necessary.
+ * In particular, typical (iTunes) m4a expects LC profile AAC data.
  *
  * <refsect2>
  * <title>Example launch line</title>
- * |[
+ * <programlisting>
  * gst-launch audiotestsrc wave=sine num-buffers=100 ! audioconvert ! faac ! matroskamux ! filesink location=sine.mkv
- * ]| Encode a sine beep as aac and write to matroska container.
+ * </programlisting>
  * </refsect2>
  */
 
@@ -48,8 +50,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gst/audio/multichannel.h>
-
 #include "gstfaac.h"
 
 #define SINK_CAPS \
@@ -79,8 +79,7 @@
     "audio/mpeg, "                     \
     "mpegversion = (int) { 4, 2 }, "   \
     "channels = (int) [ 1, 6 ], "      \
-    "rate = (int) [ 8000, 96000 ], "   \
-    "stream-format = (string) { adts, raw } "
+    "rate = (int) [ 8000, 96000 ]"
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -91,6 +90,12 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (SINK_CAPS));
 
+static const GstElementDetails gst_faac_details =
+GST_ELEMENT_DETAILS ("AAC audio encoder",
+    "Codec/Encoder/Audio",
+    "Free MPEG-2/4 AAC encoder",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
 enum
 {
   ARG_0,
@@ -116,7 +121,6 @@ static void gst_faac_get_property (GObject * object,
 static gboolean gst_faac_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_faac_configure_source_pad (GstFaac * faac);
 static gboolean gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps);
-static GstCaps *gst_faac_sink_getcaps (GstPad * pad);
 static GstFlowReturn gst_faac_push_buffers (GstFaac * faac, gboolean force);
 static GstFlowReturn gst_faac_chain (GstPad * pad, GstBuffer * data);
 static GstStateChangeReturn gst_faac_change_state (GstElement * element,
@@ -128,12 +132,6 @@ GST_DEBUG_CATEGORY_STATIC (faac_debug);
 #define GST_CAT_DEFAULT faac_debug
 
 #define FAAC_DEFAULT_MPEGVERSION 4
-#define FAAC_DEFAULT_OUTPUTFORMAT 0     /* RAW */
-#define FAAC_DEFAULT_BITRATE      128 * 1000
-#define FAAC_DEFAULT_PROFILE      LOW
-#define FAAC_DEFAULT_TNS          FALSE
-#define FAAC_DEFAULT_MIDSIDE      TRUE
-#define FAAC_DEFAULT_SHORTCTL     SHORTCTL_NORMAL
 
 GType
 gst_faac_get_type (void)
@@ -178,10 +176,7 @@ gst_faac_base_init (GstFaacClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_details_simple (element_class, "AAC audio encoder",
-      "Codec/Encoder/Audio",
-      "Free MPEG-2/4 AAC encoder",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+  gst_element_class_set_details (element_class, &gst_faac_details);
 
   GST_DEBUG_CATEGORY_INIT (faac_debug, "faac", 0, "AAC encoding");
 }
@@ -264,28 +259,24 @@ gst_faac_class_init (GstFaacClass * klass)
   /* properties */
   g_object_class_install_property (gobject_class, ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate (bps)", "Bitrate in bits/sec",
-          8 * 1000, 320 * 1000, FAAC_DEFAULT_BITRATE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          8 * 1000, 320 * 1000, 128 * 1000, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PROFILE,
       g_param_spec_enum ("profile", "Profile", "MPEG/AAC encoding profile",
-          GST_TYPE_FAAC_PROFILE, FAAC_DEFAULT_PROFILE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_FAAC_PROFILE, MAIN, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_TNS,
       g_param_spec_boolean ("tns", "TNS", "Use temporal noise shaping",
-          FAAC_DEFAULT_TNS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_MIDSIDE,
       g_param_spec_boolean ("midside", "Midside", "Allow mid/side encoding",
-          FAAC_DEFAULT_MIDSIDE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_SHORTCTL,
       g_param_spec_enum ("shortctl", "Block type",
           "Block type encorcing",
-          GST_TYPE_FAAC_SHORTCTL, FAAC_DEFAULT_SHORTCTL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_FAAC_SHORTCTL, MAIN, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_OUTPUTFORMAT,
       g_param_spec_enum ("outputformat", "Output format",
           "Format of output frames",
-          GST_TYPE_FAAC_OUTPUTFORMAT, FAAC_DEFAULT_OUTPUTFORMAT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_FAAC_OUTPUTFORMAT, 0 /* RAW */ , G_PARAM_READWRITE));
 
   /* virtual functions */
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_faac_change_state);
@@ -299,8 +290,6 @@ gst_faac_init (GstFaac * faac)
       GST_DEBUG_FUNCPTR (gst_faac_chain));
   gst_pad_set_setcaps_function (faac->sinkpad,
       GST_DEBUG_FUNCPTR (gst_faac_sink_setcaps));
-  gst_pad_set_getcaps_function (faac->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_faac_sink_getcaps));
   gst_pad_set_event_function (faac->sinkpad,
       GST_DEBUG_FUNCPTR (gst_faac_sink_event));
   gst_element_add_pad (GST_ELEMENT (faac), faac->sinkpad);
@@ -312,12 +301,12 @@ gst_faac_init (GstFaac * faac)
   faac->adapter = gst_adapter_new ();
 
   /* default properties */
-  faac->bitrate = FAAC_DEFAULT_BITRATE;
-  faac->profile = FAAC_DEFAULT_PROFILE;
-  faac->shortctl = FAAC_DEFAULT_SHORTCTL;
-  faac->outputformat = FAAC_DEFAULT_OUTPUTFORMAT;
-  faac->tns = FAAC_DEFAULT_TNS;
-  faac->midside = FAAC_DEFAULT_MIDSIDE;
+  faac->bitrate = 1000 * 128;
+  faac->profile = MAIN;
+  faac->shortctl = SHORTCTL_NORMAL;
+  faac->outputformat = 0;       /* RAW */
+  faac->tns = FALSE;
+  faac->midside = TRUE;
 
   gst_faac_reset (faac);
 }
@@ -352,82 +341,6 @@ gst_faac_close_encoder (GstFaac * faac)
   faac->offset = 0;
 }
 
-static const GstAudioChannelPosition aac_channel_positions[][8] = {
-  {GST_AUDIO_CHANNEL_POSITION_FRONT_MONO},
-  {GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-      GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT},
-  {
-        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-      },
-  {
-        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-      GST_AUDIO_CHANNEL_POSITION_REAR_CENTER},
-  {
-        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-      GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT},
-  {
-        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-      GST_AUDIO_CHANNEL_POSITION_LFE}
-};
-
-static GstCaps *
-gst_faac_sink_getcaps (GstPad * pad)
-{
-  static volatile gsize sinkcaps = 0;
-
-  if (g_once_init_enter (&sinkcaps)) {
-    GstCaps *tmp = gst_caps_new_empty ();
-    GstStructure *s, *t;
-    gint i, c;
-
-    s = gst_structure_new ("audio/x-raw-int",
-        "endianness", G_TYPE_INT, G_BYTE_ORDER,
-        "signed", G_TYPE_BOOLEAN, TRUE,
-        "width", G_TYPE_INT, 16,
-        "depth", G_TYPE_INT, 16, "rate", GST_TYPE_INT_RANGE, 8000, 96000, NULL);
-
-    for (i = 1; i <= 6; i++) {
-      GValue chanpos = { 0 };
-      GValue pos = { 0 };
-
-      t = gst_structure_copy (s);
-
-      gst_structure_set (t, "channels", G_TYPE_INT, i, NULL);
-
-      g_value_init (&chanpos, GST_TYPE_ARRAY);
-      g_value_init (&pos, GST_TYPE_AUDIO_CHANNEL_POSITION);
-
-      for (c = 0; c < i; c++) {
-        g_value_set_enum (&pos, aac_channel_positions[i - 1][c]);
-        gst_value_array_append_value (&chanpos, &pos);
-      }
-      g_value_unset (&pos);
-
-      gst_structure_set_value (t, "channel-positions", &chanpos);
-      g_value_unset (&chanpos);
-      gst_caps_append_structure (tmp, t);
-    }
-    gst_structure_free (s);
-
-    GST_DEBUG_OBJECT (pad, "Generated sinkcaps: %" GST_PTR_FORMAT, tmp);
-
-    g_once_init_leave (&sinkcaps, (gsize) tmp);
-  }
-
-  return gst_caps_ref ((GstCaps *) sinkcaps);
-}
-
 static gboolean
 gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
@@ -572,28 +485,16 @@ gst_faac_configure_source_pad (GstFaac * faac)
     conf->bitRate = maxbitrate;
   }
 
-  /* default 0 to start with, libfaac chooses based on bitrate */
-  conf->bandWidth = 0;
-
   if (!faacEncSetConfiguration (faac->handle, conf))
     goto set_failed;
 
-  /* let's see what really happened,
-   * note that this may not really match desired rate */
-  GST_DEBUG_OBJECT (faac, "average bitrate: %lu kbps",
-      (conf->bitRate + 500) / 1000 * faac->channels);
-  GST_DEBUG_OBJECT (faac, "quantization quality: %ld", conf->quantqual);
-  GST_DEBUG_OBJECT (faac, "bandwidth: %d Hz", conf->bandWidth);
-
   /* now create a caps for it all */
   srccaps = gst_caps_new_simple ("audio/mpeg",
       "mpegversion", G_TYPE_INT, mpegversion,
       "channels", G_TYPE_INT, faac->channels,
-      "rate", G_TYPE_INT, faac->samplerate,
-      "stream-format", G_TYPE_STRING, (faac->outputformat ? "adts" : "raw"),
-      NULL);
+      "rate", G_TYPE_INT, faac->samplerate, NULL);
 
-  if (!faac->outputformat) {
+  if (mpegversion == 4) {
     GstBuffer *codec_data;
     guint8 *config = NULL;
     gulong config_len = 0;
@@ -703,17 +604,17 @@ gst_faac_push_buffers (GstFaac * faac, gboolean force)
         break;
     }
 
-    /* deal with encoder lead-out */
+    /* in case encoder returns more data than is expected (which seems possible)
+     * ignore the extra part */
     if (G_UNLIKELY (av == 0 && faac->offset == 0)) {
-      GST_DEBUG_OBJECT (faac, "encoder returned additional data");
-      /* continuous with previous output, ok to have 0 duration */
-      timestamp = faac->next_ts;
-    } else {
-      /* after some caching, finally some data */
-      /* adapter gives time */
-      timestamp = gst_adapter_prev_timestamp (faac->adapter, &distance);
+      GST_DEBUG_OBJECT (faac, "encoder returned unexpected data; discarding");
+      gst_buffer_unref (outbuf);
+      continue;
     }
 
+    /* after some caching, finally some data */
+    /* adapter gives time */
+    timestamp = gst_adapter_prev_timestamp (faac->adapter, &distance);
     if (G_LIKELY ((av = gst_adapter_available (faac->adapter)) >= frame_size)) {
       /* must have then come from a complete frame */
       gst_adapter_flush (faac->adapter, frame_size);
@@ -734,9 +635,6 @@ gst_faac_push_buffers (GstFaac * faac, gboolean force)
     GST_BUFFER_DURATION (outbuf) =
         GST_FRAMES_TO_CLOCK_TIME (size / faac->channels / faac->bps,
         faac->samplerate);
-    faac->next_ts =
-        GST_BUFFER_TIMESTAMP (outbuf) + GST_BUFFER_DURATION (outbuf);
-
     /* perhaps check/set DISCONT based on timestamps ? */
 
     GST_LOG_OBJECT (faac, "Pushing out buffer time: %" GST_TIME_FORMAT
@@ -764,6 +662,7 @@ gst_faac_push_buffers (GstFaac * faac, gboolean force)
 encode_failed:
   {
     GST_ELEMENT_ERROR (faac, LIBRARY, ENCODE, (NULL), (NULL));
+    gst_buffer_unref (outbuf);
     return GST_FLOW_ERROR;
   }
 }
