@@ -38,6 +38,10 @@
 GST_DEBUG_CATEGORY_EXTERN (resindvd_debug);
 #define GST_CAT_DEFAULT resindvd_debug
 
+#define USE_VIDEOQ 0
+#define USE_HARDCODED_VIDEODEC 0
+#define USE_HARDCODED_AUDIODEC 0
+
 #define DVDBIN_LOCK(d) g_mutex_lock((d)->dvd_lock)
 #define DVDBIN_UNLOCK(d) g_mutex_unlock((d)->dvd_lock)
 
@@ -98,18 +102,21 @@ static GstStateChangeReturn rsn_dvdbin_change_state (GstElement * element,
 static void
 rsn_dvdbin_base_init (gpointer gclass)
 {
-
+  static GstElementDetails element_details = {
+    "rsndvdbin",
+    "Generic/Bin/Player",
+    "DVD playback element",
+    "Jan Schmidt <thaytan@noraisin.net>"
+  };
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &video_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &audio_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &subpicture_src_template);
-  gst_element_class_set_details_simple (element_class, "rsndvdbin",
-      "Generic/Bin/Player",
-      "DVD playback element", "Jan Schmidt <thaytan@noraisin.net>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&video_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&audio_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&subpicture_src_template));
+  gst_element_class_set_details (element_class, &element_details);
 
   element_class->change_state = GST_DEBUG_FUNCPTR (rsn_dvdbin_change_state);
 }
@@ -118,8 +125,10 @@ static void
 rsn_dvdbin_class_init (RsnDvdBinClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   gobject_class->finalize = rsn_dvdbin_finalize;
   gobject_class->set_property = rsn_dvdbin_set_property;
@@ -127,7 +136,7 @@ rsn_dvdbin_class_init (RsnDvdBinClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_DEVICE,
       g_param_spec_string ("device", "Device", "DVD device location",
-          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          NULL, G_PARAM_READWRITE));
 }
 
 static void
@@ -173,7 +182,7 @@ rsn_dvdbin_uri_get_type (void)
 static gchar **
 rsn_dvdbin_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { (char *) "dvd", NULL };
+  static gchar *protocols[] = { "dvd", NULL };
 
   return protocols;
 }
@@ -388,10 +397,17 @@ create_elements (RsnDvdBin * dvdbin)
       "max-size-time", (7 * GST_SECOND / 10), "max-size-bytes", 0,
       "max-size-buffers", 0, NULL);
 
+#if USE_HARDCODED_VIDEODEC
+  if (!try_create_piece (dvdbin, DVD_ELEM_VIDDEC, "mpeg2dec", 0, "viddec",
+          "video decoder"))
+    return FALSE;
+
+#else
   /* Decodebin will throw a missing element message to find an MPEG decoder */
   if (!try_create_piece (dvdbin, DVD_ELEM_VIDDEC, NULL, RSN_TYPE_VIDEODEC,
           "viddec", "video decoder"))
     return FALSE;
+#endif
 
   if (!try_create_piece (dvdbin, DVD_ELEM_PARSET, NULL, RSN_TYPE_RSNPARSETTER,
           "rsnparsetter", "Aspect ratio adjustment"))
@@ -407,7 +423,29 @@ create_elements (RsnDvdBin * dvdbin)
   gst_object_unref (sink);
   src = sink = NULL;
 
+#if USE_VIDEOQ
+  /* Add a small amount of queueing after the video decoder. */
+  if (!try_create_piece (dvdbin, DVD_ELEM_VIDQ, "queue", 0, "vid_q",
+          "video decoder buffer"))
+    return FALSE;
+  g_object_set (dvdbin->pieces[DVD_ELEM_VIDQ],
+      "max-size-time", G_GUINT64_CONSTANT (0), "max-size-bytes", 0,
+      "max-size-buffers", 3, NULL);
+
   src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_PARSET], "src");
+  sink = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_VIDQ], "sink");
+  if (src == NULL || sink == NULL)
+    goto failed_vidq_connect;
+  if (GST_PAD_LINK_FAILED (gst_pad_link (src, sink)))
+    goto failed_vidq_connect;
+  gst_object_unref (src);
+  gst_object_unref (sink);
+  src = sink = NULL;
+
+  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_VIDQ], "src");
+#else
+  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_PARSET], "src");
+#endif
   if (src == NULL)
     goto failed_video_ghost;
   src_templ = gst_static_pad_template_get (&video_src_template);
@@ -425,6 +463,11 @@ create_elements (RsnDvdBin * dvdbin)
   gst_object_unref (src);
   src = NULL;
 
+#if USE_HARDCODED_VIDEODEC
+  gst_element_add_pad (GST_ELEMENT (dvdbin), dvdbin->video_pad);
+  dvdbin->video_added = TRUE;
+#endif
+
   if (!try_create_piece (dvdbin, DVD_ELEM_SPU_SELECT, NULL,
           RSN_TYPE_STREAM_SELECTOR, "subpselect", "Subpicture stream selector"))
     return FALSE;
@@ -470,25 +513,22 @@ create_elements (RsnDvdBin * dvdbin)
           RSN_TYPE_STREAM_SELECTOR, "audioselect", "Audio stream selector"))
     return FALSE;
 
-  if (!try_create_piece (dvdbin, DVD_ELEM_AUD_MUNGE, NULL,
-          RSN_TYPE_AUDIOMUNGE, "audioearlymunge", "Audio output filter"))
+#if USE_HARDCODED_AUDIODEC
+  if (!try_create_piece (dvdbin, DVD_ELEM_AUDDEC, "a52dec", 0,
+          "auddec", "audio decoder"))
     return FALSE;
-
+#else
   if (!try_create_piece (dvdbin, DVD_ELEM_AUDDEC, NULL,
           RSN_TYPE_AUDIODEC, "auddec", "audio decoder"))
     return FALSE;
+#endif
 
-  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUD_MUNGE], "src");
-  sink = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUDDEC], "sink");
-  if (src == NULL || sink == NULL)
-    goto failed_aud_connect;
-  if (GST_PAD_LINK_FAILED (gst_pad_link (src, sink)))
-    goto failed_aud_connect;
-  gst_object_unref (sink);
-  gst_object_unref (src);
-  src = sink = NULL;
+  /* rsnaudiomunge goes after the audio decoding to regulate the stream */
+  if (!try_create_piece (dvdbin, DVD_ELEM_AUD_MUNGE, NULL,
+          RSN_TYPE_AUDIOMUNGE, "audiomunge", "Audio output filter"))
+    return FALSE;
 
-  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUD_SELECT], "src");
+  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUDDEC], "src");
   sink =
       gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUD_MUNGE], "sink");
   if (src == NULL || sink == NULL)
@@ -499,8 +539,18 @@ create_elements (RsnDvdBin * dvdbin)
   gst_object_unref (src);
   src = sink = NULL;
 
+  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUD_SELECT], "src");
+  sink = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUDDEC], "sink");
+  if (src == NULL || sink == NULL)
+    goto failed_aud_connect;
+  if (GST_PAD_LINK_FAILED (gst_pad_link (src, sink)))
+    goto failed_aud_connect;
+  gst_object_unref (sink);
+  gst_object_unref (src);
+  src = sink = NULL;
+
   /* ghost audio munge output pad onto bin */
-  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUDDEC], "src");
+  src = gst_element_get_static_pad (dvdbin->pieces[DVD_ELEM_AUD_MUNGE], "src");
   if (src == NULL)
     goto failed_aud_ghost;
   src_templ = gst_static_pad_template_get (&audio_src_template);
@@ -534,6 +584,12 @@ failed_viddec_connect:
   GST_ELEMENT_ERROR (dvdbin, CORE, FAILED, (NULL),
       ("Could not connect DVD video decoder and aspect ratio adjuster"));
   goto error_out;
+#if USE_VIDEOQ
+failed_vidq_connect:
+  GST_ELEMENT_ERROR (dvdbin, CORE, FAILED, (NULL),
+      ("Could not connect DVD aspect ratio adjuster and video buffer elements"));
+  goto error_out;
+#endif
 failed_video_ghost:
   GST_ELEMENT_ERROR (dvdbin, CORE, FAILED, (NULL),
       ("Could not ghost video output pad"));
@@ -700,7 +756,7 @@ demux_pad_added (GstElement * element, GstPad * pad, RsnDvdBin * dvdbin)
         gst_element_get_request_pad (dvdbin->pieces[DVD_ELEM_SPU_SELECT],
         "sink%d");
     skip_mq = TRUE;
-  } else if (can_sink_caps (dvdbin->pieces[DVD_ELEM_AUD_MUNGE], caps)) {
+  } else if (can_sink_caps (dvdbin->pieces[DVD_ELEM_AUDDEC], caps)) {
     GST_LOG_OBJECT (dvdbin, "Found audio pad w/ caps %" GST_PTR_FORMAT, caps);
     dest_pad =
         gst_element_get_request_pad (dvdbin->pieces[DVD_ELEM_AUD_SELECT],
@@ -719,7 +775,7 @@ demux_pad_added (GstElement * element, GstPad * pad, RsnDvdBin * dvdbin)
           ("No MPEG video decoder found"));
     } else {
       GST_ELEMENT_WARNING (dvdbin, STREAM, CODEC_NOT_FOUND, (NULL),
-          ("No MPEG audio decoder found"));
+          ("No MPEG video decoder found"));
     }
   }
 
