@@ -17,10 +17,8 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
+#define FLOAT_SAMPLES 1
+#include <soundtouch/SoundTouch.h>
 /* FIXME: workaround for SoundTouch.h of version 1.3.1 defining those
  * variables while it shouldn't. */
 #undef VERSION
@@ -31,8 +29,9 @@
 #undef PACKAGE_BUGREPORT
 #undef PACKAGE
 
-#define FLOAT_SAMPLES 1
-#include <soundtouch/SoundTouch.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <gst/gst.h>
 #include <gst/controller/gstcontroller.h>
@@ -55,7 +54,6 @@ struct _GstPitchPrivate
 enum
 {
   ARG_0,
-  ARG_OUT_RATE,
   ARG_RATE,
   ARG_TEMPO,
   ARG_PITCH
@@ -106,10 +104,10 @@ gst_pitch_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_pitch_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_pitch_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_pitch_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_pitch_sink_template));
 
   gst_element_class_set_details_simple (gstelement_class, "Pitch controller",
       "Filter/Converter/Audio", "Control the pitch of an audio stream",
@@ -136,22 +134,17 @@ gst_pitch_class_init (GstPitchClass * klass)
   g_object_class_install_property (gobject_class, ARG_PITCH,
       g_param_spec_float ("pitch", "Pitch",
           "Audio stream pitch", 0.1, 10.0, 1.0,
-          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS)));
+          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE)));
 
   g_object_class_install_property (gobject_class, ARG_TEMPO,
       g_param_spec_float ("tempo", "Tempo",
           "Audio stream tempo", 0.1, 10.0, 1.0,
-          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS)));
+          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE)));
 
   g_object_class_install_property (gobject_class, ARG_RATE,
       g_param_spec_float ("rate", "Rate",
           "Audio stream rate", 0.1, 10.0, 1.0,
-          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, ARG_OUT_RATE,
-      g_param_spec_float ("output-rate", "Output Rate",
-          "Output rate on downstream segment events", 0.1, 10.0, 1.0,
-          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS)));
+          (GParamFlags) (G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE)));
 
   g_type_class_add_private (gobject_class, sizeof (GstPitchPrivate));
 }
@@ -188,18 +181,18 @@ gst_pitch_init (GstPitch * pitch, GstPitchClass * pitch_class)
       GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_element_add_pad (GST_ELEMENT (pitch), pitch->srcpad);
 
+  gst_element_no_more_pads (GST_ELEMENT (pitch));
+
   pitch->priv->st = new soundtouch::SoundTouch ();
 
   pitch->tempo = 1.0;
   pitch->rate = 1.0;
-  pitch->out_seg_rate = 1.0;
-  pitch->seg_arate = 1.0;
   pitch->pitch = 1.0;
   pitch->next_buffer_time = 0;
   pitch->next_buffer_offset = 0;
 
   pitch->priv->st->setRate (pitch->rate);
-  pitch->priv->st->setTempo (pitch->tempo * pitch->seg_arate);
+  pitch->priv->st->setTempo (pitch->tempo);
   pitch->priv->st->setPitch (pitch->pitch);
 
   pitch->priv->stream_time_ratio = 1.0;
@@ -240,22 +233,18 @@ gst_pitch_set_property (GObject * object, guint prop_id,
   switch (prop_id) {
     case ARG_TEMPO:
       pitch->tempo = g_value_get_float (value);
-      pitch->priv->stream_time_ratio = pitch->tempo * pitch->rate * pitch->seg_arate;
-      pitch->priv->st->setTempo (pitch->tempo * pitch->seg_arate);
+      pitch->priv->stream_time_ratio = pitch->tempo * pitch->rate;
+      pitch->priv->st->setTempo (pitch->tempo);
       GST_OBJECT_UNLOCK (pitch);
       gst_pitch_update_duration (pitch);
       break;
     case ARG_RATE:
       pitch->rate = g_value_get_float (value);
-      pitch->priv->stream_time_ratio = pitch->tempo * pitch->rate * pitch->seg_arate;
+      pitch->priv->stream_time_ratio = pitch->tempo * pitch->rate;
       pitch->priv->st->setRate (pitch->rate);
       GST_OBJECT_UNLOCK (pitch);
       gst_pitch_update_duration (pitch);
       break;
-    case ARG_OUT_RATE:
-      /* Has no effect until the next input segment */
-      pitch->out_seg_rate = g_value_get_float (value);
-      GST_OBJECT_UNLOCK (pitch);
     case ARG_PITCH:
       pitch->pitch = g_value_get_float (value);
       pitch->priv->st->setPitch (pitch->pitch);
@@ -282,9 +271,6 @@ gst_pitch_get_property (GObject * object, guint prop_id,
     case ARG_RATE:
       g_value_set_float (value, pitch->rate);
       break;
-    case ARG_OUT_RATE:
-      g_value_set_float (value, pitch->out_seg_rate);
-      break;
     case ARG_PITCH:
       g_value_set_float (value, pitch->pitch);
       break;
@@ -682,6 +668,10 @@ gst_pitch_src_query (GstPad * pad, GstQuery * query)
           GST_DEBUG ("Calculated total latency : min %"
               GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
               GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+          g_print ("Calculated total latency : min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
           gst_query_set_latency (query, live, min, max);
         }
         gst_object_unref (peer);
@@ -710,17 +700,16 @@ gst_pitch_process_segment (GstPitch * pitch, GstEvent ** event)
   gint64 start_value, stop_value, base;
   gint64 next_offset = 0, next_time = 0;
   gboolean update = FALSE;
-  gdouble rate, out_seg_rate, arate, our_arate;
+  gdouble rate;
   gfloat stream_time_ratio;
 
   g_return_val_if_fail (event, FALSE);
 
   GST_OBJECT_LOCK (pitch);
   stream_time_ratio = pitch->priv->stream_time_ratio;
-  out_seg_rate = pitch->out_seg_rate;
   GST_OBJECT_UNLOCK (pitch);
 
-  gst_event_parse_new_segment_full (*event, &update, &rate, &arate, &format, &start_value,
+  gst_event_parse_new_segment (*event, &update, &rate, &format, &start_value,
       &stop_value, &base);
 
   if (format != GST_FORMAT_TIME && format != GST_FORMAT_DEFAULT) {
@@ -729,36 +718,21 @@ gst_pitch_process_segment (GstPitch * pitch, GstEvent ** event)
         "open ended NEWSEGMENT in TIME format.");
     gst_event_unref (*event);
     *event =
-        gst_event_new_new_segment_full (update, rate, arate, GST_FORMAT_TIME, 0, -1, 0);
+        gst_event_new_new_segment (update, rate, GST_FORMAT_TIME, 0, -1, 0);
     start_value = 0;
     stop_value = -1;
     base = 0;
   }
 
-  /* Figure out how much of the incoming 'rate' we'll apply ourselves */
-  our_arate = rate / out_seg_rate;
-  /* update the output rate variables */
-  rate = out_seg_rate;
-  arate *= our_arate;
-
   GST_LOG_OBJECT (pitch->sinkpad,
       "segment %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT " (%d)", start_value,
       stop_value, format);
 
-  stream_time_ratio = pitch->tempo * pitch->rate * pitch->seg_arate;
-
   if (stream_time_ratio == 0) {
     GST_LOG_OBJECT (pitch->sinkpad, "stream_time_ratio is zero");
     return FALSE;
   }
 
-  /* Update the playback rate */
-  GST_OBJECT_LOCK (pitch);
-  pitch->seg_arate = our_arate;
-  pitch->priv->stream_time_ratio = stream_time_ratio;
-  pitch->priv->st->setTempo (pitch->tempo * pitch->seg_arate);
-  GST_OBJECT_UNLOCK (pitch);
-
   start_value = (gint64) (start_value / stream_time_ratio);
   if (stop_value != -1)
     stop_value = (gint64) (stop_value / stream_time_ratio);
@@ -783,8 +757,8 @@ gst_pitch_process_segment (GstPitch * pitch, GstEvent ** event)
   pitch->next_buffer_offset = next_offset;
 
   gst_event_unref (*event);
-  *event = gst_event_new_new_segment_full (update, rate, arate, format,
-       start_value, stop_value, base);
+  *event = gst_event_new_new_segment (update, rate, format, start_value,
+      stop_value, base);
 
   return TRUE;
 }
@@ -803,8 +777,6 @@ gst_pitch_sink_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_FLUSH_STOP:
       gst_pitch_flush_buffer (pitch, FALSE);
       pitch->priv->st->clear ();
-      pitch->next_buffer_offset = 0;
-      pitch->next_buffer_time = 0;
       pitch->min_latency = pitch->max_latency = 0;
       break;
     case GST_EVENT_EOS:
