@@ -37,14 +37,13 @@
  *    the buffer
  */
 
+#include <stdlib.h>
+#include <string.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include <stdlib.h>
-#include <string.h>
-
-#include <gst/controller/gstcontroller.h>
 #include <gst/audio/audio.h>
 #include "gstsignalprocessor.h"
 
@@ -97,7 +96,8 @@ gst_signal_processor_pad_template_get_type (void)
  */
 void
 gst_signal_processor_class_add_pad_template (GstSignalProcessorClass * klass,
-    const gchar * name, GstPadDirection direction, guint index, guint channels)
+    const gchar * name, GstPadDirection direction, guint index, guint channels,
+    const GstAudioChannelPosition * pos)
 {
   GstPadTemplate *new;
   GstCaps *caps;
@@ -108,20 +108,19 @@ gst_signal_processor_class_add_pad_template (GstSignalProcessorClass * klass,
 
   caps = gst_caps_new_simple ("audio/x-raw-float",
       "endianness", G_TYPE_INT, G_BYTE_ORDER,
-      "width", G_TYPE_INT, 32, "channels", G_TYPE_INT, channels,
-      "rate", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
+      "width", G_TYPE_INT, 32, "channels", G_TYPE_INT, channels, NULL);
+
+  if (pos)
+    gst_audio_set_caps_channel_positions_list (caps, pos, channels);
 
   new = g_object_new (GST_TYPE_SIGNAL_PROCESSOR_PAD_TEMPLATE,
       "name", name, "name-template", name,
       "direction", direction, "presence", GST_PAD_ALWAYS, "caps", caps, NULL);
 
-  gst_caps_unref (caps);
-
   GST_SIGNAL_PROCESSOR_PAD_TEMPLATE (new)->index = index;
   GST_SIGNAL_PROCESSOR_PAD_TEMPLATE (new)->channels = channels;
 
   gst_element_class_add_pad_template (GST_ELEMENT_CLASS (klass), new);
-  g_object_unref (new);
 }
 
 
@@ -169,8 +168,8 @@ gst_signal_processor_pad_get_type (void)
 static void
 _do_init (GType object_type)
 {
-  GST_DEBUG_CATEGORY_INIT (gst_signal_processor_debug, "signalprocessor", 0,
-      "signalprocessor baseclass");
+  GST_DEBUG_CATEGORY_INIT (gst_signal_processor_debug, "gst-dsp", 0,
+      "signalprocessor element");
 }
 
 GST_BOILERPLATE_FULL (GstSignalProcessor, gst_signal_processor, GstElement,
@@ -212,47 +211,44 @@ gst_signal_processor_class_init (GstSignalProcessorClass * klass)
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_signal_processor_change_state);
-
-  GST_DEBUG_REGISTER_FUNCPTR (gst_signal_processor_setcaps);
-  GST_DEBUG_REGISTER_FUNCPTR (gst_signal_processor_event);
-  GST_DEBUG_REGISTER_FUNCPTR (gst_signal_processor_chain);
-  GST_DEBUG_REGISTER_FUNCPTR (gst_signal_processor_sink_activate_push);
-  GST_DEBUG_REGISTER_FUNCPTR (gst_signal_processor_getrange);
-  GST_DEBUG_REGISTER_FUNCPTR (gst_signal_processor_src_activate_pull);
 }
 
 static void
 gst_signal_processor_add_pad_from_template (GstSignalProcessor * self,
     GstPadTemplate * templ)
 {
-  GstPad *pad;
+  GstPad *new;
 
-  pad = g_object_new (GST_TYPE_SIGNAL_PROCESSOR_PAD,
+  new = g_object_new (GST_TYPE_SIGNAL_PROCESSOR_PAD,
       "name", GST_OBJECT_NAME (templ), "direction", templ->direction,
       "template", templ, NULL);
-  GST_SIGNAL_PROCESSOR_PAD (pad)->index =
+  GST_SIGNAL_PROCESSOR_PAD (new)->index =
       GST_SIGNAL_PROCESSOR_PAD_TEMPLATE (templ)->index;
-  GST_SIGNAL_PROCESSOR_PAD (pad)->channels =
+  GST_SIGNAL_PROCESSOR_PAD (new)->channels =
       GST_SIGNAL_PROCESSOR_PAD_TEMPLATE (templ)->channels;
 
-  gst_pad_set_setcaps_function (pad, gst_signal_processor_setcaps);
+  gst_pad_set_setcaps_function (new,
+      GST_DEBUG_FUNCPTR (gst_signal_processor_setcaps));
 
   if (templ->direction == GST_PAD_SINK) {
-    GST_DEBUG_OBJECT (pad, "added new sink pad");
+    GST_DEBUG ("added new sink pad");
 
-    gst_pad_set_event_function (pad, gst_signal_processor_event);
-    gst_pad_set_chain_function (pad, gst_signal_processor_chain);
-    gst_pad_set_activatepush_function (pad,
-        gst_signal_processor_sink_activate_push);
+    gst_pad_set_event_function (new,
+        GST_DEBUG_FUNCPTR (gst_signal_processor_event));
+    gst_pad_set_chain_function (new,
+        GST_DEBUG_FUNCPTR (gst_signal_processor_chain));
+    gst_pad_set_activatepush_function (new,
+        GST_DEBUG_FUNCPTR (gst_signal_processor_sink_activate_push));
   } else {
-    GST_DEBUG_OBJECT (pad, "added new src pad");
+    GST_DEBUG ("added new src pad");
 
-    gst_pad_set_getrange_function (pad, gst_signal_processor_getrange);
-    gst_pad_set_activatepull_function (pad,
-        gst_signal_processor_src_activate_pull);
+    gst_pad_set_getrange_function (new,
+        GST_DEBUG_FUNCPTR (gst_signal_processor_getrange));
+    gst_pad_set_activatepull_function (new,
+        GST_DEBUG_FUNCPTR (gst_signal_processor_src_activate_pull));
   }
 
-  gst_element_add_pad (GST_ELEMENT (self), pad);
+  gst_element_add_pad (GST_ELEMENT (self), new);
 }
 
 static void
@@ -282,6 +278,8 @@ gst_signal_processor_init (GstSignalProcessor * self,
   /* init */
   self->pending_in = klass->num_group_in + klass->num_audio_in;
   self->pending_out = 0;
+
+  self->sample_rate = 0;
 }
 
 static void
@@ -306,7 +304,7 @@ gst_signal_processor_finalize (GObject * object)
 }
 
 static gboolean
-gst_signal_processor_setup (GstSignalProcessor * self, GstCaps * caps)
+gst_signal_processor_setup (GstSignalProcessor * self, guint sample_rate)
 {
   GstSignalProcessorClass *klass;
   gboolean ret = TRUE;
@@ -318,7 +316,7 @@ gst_signal_processor_setup (GstSignalProcessor * self, GstCaps * caps)
   g_return_val_if_fail (self->state == GST_SIGNAL_PROCESSOR_STATE_NULL, FALSE);
 
   if (klass->setup)
-    ret = klass->setup (self, caps);
+    ret = klass->setup (self, sample_rate);
 
   if (!ret)
     goto setup_failed;
@@ -329,7 +327,7 @@ gst_signal_processor_setup (GstSignalProcessor * self, GstCaps * caps)
 
 setup_failed:
   {
-    GST_INFO_OBJECT (self, "setup() failed for caps: %" GST_PTR_FORMAT, caps);
+    GST_INFO_OBJECT (self, "setup() failed at %u Hz", sample_rate);
     return ret;
   }
 }
@@ -479,21 +477,27 @@ gst_signal_processor_setcaps (GstPad * pad, GstCaps * caps)
      implementations know */
   if (!gst_caps_is_equal (caps, self->caps)) {
     GstStructure *s;
+    gint sample_rate;
 
     GST_DEBUG_OBJECT (pad, "got caps %" GST_PTR_FORMAT, caps);
 
+    s = gst_caps_get_structure (caps, 0);
+    if (!gst_structure_get_int (s, "rate", &sample_rate)) {
+      GST_WARNING ("got no sample-rate");
+      goto impossible;
+    }
+
+    GST_DEBUG_OBJECT (self, "Got rate=%d", sample_rate);
+
     if (GST_SIGNAL_PROCESSOR_IS_RUNNING (self))
       gst_signal_processor_stop (self);
     if (GST_SIGNAL_PROCESSOR_IS_INITIALIZED (self))
       gst_signal_processor_cleanup (self);
 
-    s = gst_caps_get_structure (caps, 0);
-    if (!gst_structure_get_int (s, "rate", &self->sample_rate))
-      goto no_sample_rate;
-
-    if (!gst_signal_processor_setup (self, caps))
+    if (!gst_signal_processor_setup (self, sample_rate))
       goto start_or_setup_failed;
 
+    self->sample_rate = sample_rate;
     gst_caps_replace (&self->caps, caps);
   } else {
     GST_DEBUG_OBJECT (self, "skipping, have caps already");
@@ -505,7 +509,7 @@ gst_signal_processor_setcaps (GstPad * pad, GstCaps * caps)
      sample rate (e.g., when having gone PLAYING->READY->PLAYING). make sure
      when we leave that the processor is RUNNING. */
   if (!GST_SIGNAL_PROCESSOR_IS_INITIALIZED (self)
-      && !gst_signal_processor_setup (self, self->caps))
+      && !gst_signal_processor_setup (self, self->sample_rate))
     goto start_or_setup_failed;
   if (!GST_SIGNAL_PROCESSOR_IS_RUNNING (self)
       && !gst_signal_processor_start (self))
@@ -515,27 +519,25 @@ gst_signal_processor_setcaps (GstPad * pad, GstCaps * caps)
 
   return TRUE;
 
-no_sample_rate:
-  {
-    GST_WARNING_OBJECT (self, "got no sample-rate");
-    gst_object_unref (self);
-    return FALSE;
-  }
 start_or_setup_failed:
   {
-    GST_WARNING_OBJECT (self, "start or setup failed");
     gst_object_unref (self);
     return FALSE;
   }
 setcaps_pull_failed:
   {
-    GST_WARNING_OBJECT (self, "activating in pull-mode failed");
+    gst_object_unref (self);
+    return FALSE;
+  }
+impossible:
+  {
+    g_critical ("something impossible happened");
     gst_object_unref (self);
     return FALSE;
   }
 }
 
-/* De-interleave a pad (gstreamer => plugin) */
+/** De-interleave a pad (gstreamer => plugin) */
 static void
 gst_signal_processor_deinterleave_group (GstSignalProcessorGroup * group,
     guint nframes)
@@ -550,7 +552,7 @@ gst_signal_processor_deinterleave_group (GstSignalProcessorGroup * group,
           = group->interleaved_buffer[(i * group->channels) + j];
 }
 
-/* Interleave a pad (plugin => gstreamer) */
+/** Interleave a pad (plugin => gstreamer) */
 static void
 gst_signal_processor_interleave_group (GstSignalProcessorGroup * group,
     guint nframes)
@@ -575,8 +577,6 @@ gst_signal_processor_event (GstPad * pad, GstEvent * event)
   self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
   bclass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
-  GST_DEBUG_OBJECT (pad, "got event %s", GST_EVENT_TYPE_NAME (event));
-
   /* FIXME, this probably isn't the correct interface: what about return values,
    * what about overriding event_default
    * Sync with GstBaseTransform::gst_base_transform_sink_event */
@@ -600,15 +600,6 @@ gst_signal_processor_event (GstPad * pad, GstEvent * event)
   return ret;
 }
 
-/*
- * gst_signal_processor_prepare:
- * @self: the element
- * nframes: wanted sample frames
- *
- * Checks if wan
- *
- * Returns: available sample frames
- */
 static guint
 gst_signal_processor_prepare (GstSignalProcessor * self, guint nframes)
 {
@@ -617,8 +608,6 @@ gst_signal_processor_prepare (GstSignalProcessor * self, guint nframes)
   GList *sinks, *srcs;
   guint samples_avail = nframes;
   guint i, in_group_index = 0, out_group_index = 0;
-  gboolean is_gap = FALSE;
-  GstClockTime ts, tss = GST_CLOCK_TIME_NONE, tse = GST_CLOCK_TIME_NONE;
 
   klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
@@ -652,40 +641,13 @@ gst_signal_processor_prepare (GstSignalProcessor * self, guint nframes)
     }
   }
 
-  GST_LOG_OBJECT (self, "want %u samples, have %u samples", nframes,
-      samples_avail);
-
   /* FIXME: return if samples_avail==0 ? */
 
-  if ((sinks = elem->sinkpads)) {
-    is_gap = TRUE;
-    while (sinks) {
-      GstSignalProcessorPad *sinkpad = (GstSignalProcessorPad *) sinks->data;
-
-      is_gap &= GST_BUFFER_FLAG_IS_SET (sinkpad->pen, GST_BUFFER_FLAG_GAP);
-      ts = GST_BUFFER_TIMESTAMP (sinkpad->pen);
-      if (GST_CLOCK_TIME_IS_VALID (ts)) {
-        tss = !GST_CLOCK_TIME_IS_VALID (tss) ? ts : MIN (tss, ts);
-        tse = !GST_CLOCK_TIME_IS_VALID (tse) ? ts : MAX (tse, ts);
-      }
-      sinks = sinks->next;
-    }
-    ts = (tss == tse) ? tss : GST_CLOCK_TIME_NONE;
-    GST_LOG_OBJECT (self, "is gap: %d, tss %" GST_TIME_FORMAT ", tse %"
-        GST_TIME_FORMAT, is_gap, GST_TIME_ARGS (tss), GST_TIME_ARGS (tse));
-  } else {
-    /* FIXME: calculate own timestamps */
-    ts = GST_CLOCK_TIME_NONE;
-  }
-
   /* now assign output buffers. we can avoid allocation by reusing input
      buffers, but only if process() can work in place, and if the input buffer
      is the exact size of the number of samples we are processing. */
   sinks = elem->sinkpads;
   srcs = elem->srcpads;
-
-  GST_LOG_OBJECT (self, "allocating %d buffers", g_list_length (srcs));
-
   if (GST_SIGNAL_PROCESSOR_CLASS_CAN_PROCESS_IN_PLACE (klass)) {
     while (sinks && srcs) {
       GstSignalProcessorPad *sinkpad, *srcpad;
@@ -719,15 +681,12 @@ gst_signal_processor_prepare (GstSignalProcessor * self, guint nframes)
 
     srcpad = (GstSignalProcessorPad *) srcs->data;
 
-    ret = gst_pad_alloc_buffer_and_set_caps (GST_PAD (srcpad),
-        GST_CLOCK_TIME_NONE,
+    ret =
+        gst_pad_alloc_buffer_and_set_caps (GST_PAD (srcpad), -1,
         samples_avail * srcpad->channels * sizeof (gfloat), self->caps,
         &srcpad->pen);
 
     if (ret != GST_FLOW_OK) {
-      GST_INFO_OBJECT (self,
-          "allocating buffer for %u samples, %u channels failed", samples_avail,
-          srcpad->channels);
       self->flow_state = ret;
       return 0;
     } else if (srcpad->channels > 1) {
@@ -738,6 +697,7 @@ gst_signal_processor_prepare (GstSignalProcessor * self, guint nframes)
         group->buffer =
             (gfloat *) g_realloc (group->buffer,
             samples_avail * srcpad->channels * sizeof (gfloat));
+      g_assert (group->buffer);
       group->channels = srcpad->channels;
       group->nframes = samples_avail;
       self->pending_out++;
@@ -746,18 +706,9 @@ gst_signal_processor_prepare (GstSignalProcessor * self, guint nframes)
       self->pending_out++;
     }
 
-    /* set time stamp */
-    GST_BUFFER_TIMESTAMP (srcpad->pen) = ts;
-    /* FIXME: handle gap flag ? */
-
     srcs = srcs->next;
   }
 
-  /* update controlled parameters */
-  if (samples_avail && GST_CLOCK_TIME_IS_VALID (ts)) {
-    gst_object_sync_values ((GObject *) self, ts);
-  }
-
   return samples_avail;
 }
 
@@ -803,14 +754,17 @@ gst_signal_processor_update_outputs (GstSignalProcessor * self,
     gst_signal_processor_interleave_group (&self->group_out[i], nprocessed);
 }
 
-static gboolean
+static void
 gst_signal_processor_process (GstSignalProcessor * self, guint nframes)
 {
+  GstElement *elem;
   GstSignalProcessorClass *klass;
 
   /* check if we have buffers enqueued */
-  g_return_val_if_fail (self->pending_in == 0, FALSE);
-  g_return_val_if_fail (self->pending_out == 0, FALSE);
+  g_return_if_fail (self->pending_in == 0);
+  g_return_if_fail (self->pending_out == 0);
+
+  elem = GST_ELEMENT (self);
 
   /* check how much input is available and prepare output buffers */
   nframes = gst_signal_processor_prepare (self, nframes);
@@ -826,14 +780,13 @@ gst_signal_processor_process (GstSignalProcessor * self, guint nframes)
   gst_signal_processor_update_inputs (self, nframes);
   gst_signal_processor_update_outputs (self, nframes);
 
-  return TRUE;
+  return;
 
 flow_error:
   {
-    GST_WARNING_OBJECT (self,
-        "gst_signal_processor_prepare() returned %d, flow_ref=%s", nframes,
-        gst_flow_get_name (self->flow_state));
-    return FALSE;
+    GST_WARNING ("gst_pad_alloc_buffer_and_set_caps() returned %d",
+        self->flow_state);
+    return;
   }
 }
 
@@ -1026,14 +979,12 @@ gst_signal_processor_chain (GstPad * pad, GstBuffer * buffer)
 
   self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
 
-  GST_LOG_OBJECT (self, "chain(%s:%s, %p) : p_in=%u, p_out=%u",
-      GST_DEBUG_PAD_NAME (pad), buffer, self->pending_in, self->pending_out);
-
   gst_signal_processor_pen_buffer (self, pad, buffer);
 
   if (self->pending_in == 0) {
-    if (gst_signal_processor_process (self, G_MAXUINT))
-      gst_signal_processor_do_pushes (self);
+    gst_signal_processor_process (self, G_MAXUINT);
+
+    gst_signal_processor_do_pushes (self);
   }
 
   gst_object_unref (self);
@@ -1046,8 +997,10 @@ gst_signal_processor_sink_activate_push (GstPad * pad, gboolean active)
 {
   gboolean result = TRUE;
   GstSignalProcessor *self;
+  GstSignalProcessorClass *bclass;
 
   self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
+  bclass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
   if (active) {
     if (self->mode == GST_ACTIVATE_NONE) {
@@ -1083,8 +1036,10 @@ gst_signal_processor_src_activate_pull (GstPad * pad, gboolean active)
 {
   gboolean result = TRUE;
   GstSignalProcessor *self;
+  GstSignalProcessorClass *bclass;
 
   self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
+  bclass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
   if (active) {
     if (self->mode == GST_ACTIVATE_NONE) {
