@@ -156,9 +156,7 @@ static void id3v2_frame_finish (GstId3v2Tag * tag, GstId3v2Frame * frame);
 static guint id3v2_frame_get_size (GstId3v2Tag * tag, GstId3v2Frame * frame);
 
 static void id3v2_tag_add_text_frame (GstId3v2Tag * tag,
-    const gchar * frame_id, const gchar ** strings, int num_strings);
-static void id3v2_tag_add_simple_text_frame (GstId3v2Tag * tag,
-    const gchar * frame_id, const gchar * string);
+    const gchar * frame_id, gchar ** strings, int num_strings);
 
 static gboolean
 id3v2_tag_init (GstId3v2Tag * tag, guint major_version)
@@ -376,11 +374,10 @@ id3v2_frame_write_string (GstId3v2Frame * frame, int encoding,
 
     /* Write the BOM */
     id3v2_frame_write_bytes (frame, (const guint8 *) bom, 2);
-    id3v2_frame_write_bytes (frame, (const guint8 *) utf16, utf16len);
-    if (null_terminate) {
-      /* NUL terminator is 2 bytes, if present. */
-      id3v2_frame_write_uint16 (frame, 0);
-    }
+    /* NUL terminator is 2 bytes, if present */
+    terminator_length = null_terminate ? 2 : 0;
+    id3v2_frame_write_bytes (frame, (const guint8 *) utf16,
+        utf16len + terminator_length);
 
     g_free (utf16);
   } else {
@@ -393,7 +390,7 @@ id3v2_frame_write_string (GstId3v2Frame * frame, int encoding,
 
 static void
 id3v2_tag_add_text_frame (GstId3v2Tag * tag, const gchar * frame_id,
-    const gchar ** strings_utf8, int num_strings)
+    gchar ** strings_utf8, int num_strings)
 {
   GstId3v2Frame frame;
   guint len, i;
@@ -436,27 +433,20 @@ id3v2_tag_add_text_frame (GstId3v2Tag * tag, const gchar * frame_id,
   g_array_append_val (tag->frames, frame);
 }
 
-static void
-id3v2_tag_add_simple_text_frame (GstId3v2Tag * tag, const gchar * frame_id,
-    const gchar * string)
-{
-  id3v2_tag_add_text_frame (tag, frame_id, (const gchar **) &string, 1);
-}
-
 /* ====================================================================== */
 
 static void
 add_text_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * frame_id)
 {
-  const gchar **strings;
+  gchar **strings;
   guint n, i;
 
   GST_LOG ("Adding '%s' frame", frame_id);
 
-  strings = g_new0 (const gchar *, num_tags + 1);
+  strings = g_new0 (gchar *, num_tags + 1);
   for (n = 0, i = 0; n < num_tags; ++n) {
-    if (gst_tag_list_peek_string_index (list, tag, n, &strings[i]) &&
+    if (gst_tag_list_get_string_index (list, tag, n, &strings[i]) &&
         strings[i] != NULL) {
       GST_LOG ("%s: %s[%u] = '%s'", frame_id, tag, i, strings[i]);
       ++i;
@@ -469,7 +459,7 @@ add_text_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     GST_WARNING ("Empty list for tag %s, skipping", tag);
   }
 
-  g_free (strings);
+  g_strfreev (strings);
 }
 
 static void
@@ -576,7 +566,7 @@ add_count_or_num_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
       GST_DEBUG ("Setting %s to %s (frame_id = %s)", tag, tag_str, frame_id);
 
-      id3v2_tag_add_simple_text_frame (id3v2tag, frame_id, tag_str);
+      id3v2_tag_add_text_frame (id3v2tag, frame_id, &tag_str, 1);
       g_free (tag_str);
     }
   } else if (corr[idx].corr_count == NULL) {
@@ -589,7 +579,7 @@ add_count_or_num_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
       gchar *tag_str = g_strdup_printf ("0/%u", count);
       GST_DEBUG ("Setting %s to %s (frame_id = %s)", tag, tag_str, frame_id);
 
-      id3v2_tag_add_simple_text_frame (id3v2tag, frame_id, tag_str);
+      id3v2_tag_add_text_frame (id3v2tag, frame_id, &tag_str, 1);
       g_free (tag_str);
     }
   }
@@ -599,31 +589,6 @@ add_count_or_num_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   }
 }
 
-static void
-add_bpm_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
-    const gchar * tag, guint num_tags, const gchar * unused)
-{
-  gdouble bpm;
-
-  GST_LOG ("Adding BPM frame");
-
-  if (gst_tag_list_get_double (list, tag, &bpm)) {
-    gchar *tag_str;
-
-    /* bpm is stored as an integer in id3 tags, but is a double in
-     * tag lists.
-     */
-    tag_str = g_strdup_printf ("%u", (guint) bpm);
-    GST_DEBUG ("Setting %s to %s", tag, tag_str);
-    id3v2_tag_add_simple_text_frame (id3v2tag, "TBPM", tag_str);
-    g_free (tag_str);
-  }
-
-  if (num_tags > 1) {
-    GST_WARNING ("more than one %s, can only handle one", tag);
-  }
-}
-
 static void
 add_comment_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * unused)
@@ -632,9 +597,9 @@ add_comment_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
   GST_LOG ("Adding comment frames");
   for (n = 0; n < num_tags; ++n) {
-    const gchar *s = NULL;
+    gchar *s = NULL;
 
-    if (gst_tag_list_peek_string_index (list, tag, n, &s) && s != NULL) {
+    if (gst_tag_list_get_string_index (list, tag, n, &s) && s != NULL) {
       gchar *desc = NULL, *val = NULL, *lang = NULL;
       int desclen, vallen, encoding1, encoding2, encoding;
       GstId3v2Frame frame;
@@ -677,6 +642,7 @@ add_comment_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
       g_array_append_val (id3v2tag->frames, frame);
     }
+    g_free (s);
   }
 }
 
@@ -777,9 +743,9 @@ add_musicbrainz_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   g_assert (idx < G_N_ELEMENTS (mb_ids));
 
   for (i = 0; i < num_tags; ++i) {
-    const gchar *id_str;
+    gchar *id_str;
 
-    if (gst_tag_list_peek_string_index (list, tag, 0, &id_str) && id_str) {
+    if (gst_tag_list_get_string_index (list, tag, 0, &id_str) && id_str) {
       /* add two frames, one with the ID the musicbrainz.org spec mentions
        * and one with the ID that applications use in the real world */
       GstId3v2Frame frame1, frame2;
@@ -800,6 +766,8 @@ add_musicbrainz_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
           mb_ids[idx].realworld_id, TRUE);
       id3v2_frame_write_string (&frame2, encoding, id_str, FALSE);
       g_array_append_val (id3v2tag->frames, frame2);
+
+      g_free (id_str);
     }
   }
 }
@@ -809,9 +777,9 @@ add_unique_file_id_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * unused)
 {
   const gchar *origin = "http://musicbrainz.org";
-  const gchar *id_str = NULL;
+  gchar *id_str = NULL;
 
-  if (gst_tag_list_peek_string_index (list, tag, 0, &id_str) && id_str) {
+  if (gst_tag_list_get_string_index (list, tag, 0, &id_str) && id_str) {
     GstId3v2Frame frame;
 
     GST_LOG ("Adding %s (%s): %s", tag, origin, id_str);
@@ -822,6 +790,8 @@ add_unique_file_id_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     id3v2_frame_write_bytes (&frame, (const guint8 *) id_str,
         strlen (id_str) + 1);
     g_array_append_val (id3v2tag->frames, frame);
+
+    g_free (id_str);
   }
 }
 
@@ -864,7 +834,7 @@ add_date_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   }
 
   if (strings[0] != NULL) {
-    id3v2_tag_add_text_frame (id3v2tag, frame_id, (const gchar **) strings, i);
+    id3v2_tag_add_text_frame (id3v2tag, frame_id, strings, i);
   } else {
     GST_WARNING ("Empty list for tag %s, skipping", tag);
   }
@@ -886,9 +856,9 @@ add_encoder_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
   strings = g_new0 (gchar *, num_tags + 1);
   for (n = 0; n < num_tags; ++n) {
-    const gchar *encoder = NULL;
+    gchar *encoder = NULL;
 
-    if (gst_tag_list_peek_string_index (list, tag, n, &encoder) && encoder) {
+    if (gst_tag_list_get_string_index (list, tag, n, &encoder) && encoder) {
       guint encoder_version;
       gchar *s;
 
@@ -902,11 +872,12 @@ add_encoder_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
       GST_LOG ("encoder[%u] = '%s'", n, s);
       strings[i] = s;
       i++;
+      g_free (encoder);
     }
   }
 
   if (strings[0] != NULL) {
-    id3v2_tag_add_text_frame (id3v2tag, "TSSE", (const gchar **) strings, i);
+    id3v2_tag_add_text_frame (id3v2tag, "TSSE", strings, i);
   } else {
     GST_WARNING ("Empty list for tag %s, skipping", tag);
   }
@@ -918,12 +889,12 @@ static void
 add_uri_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * frame_id)
 {
-  const gchar *url = NULL;
+  gchar *url = NULL;
 
   g_assert (frame_id != NULL);
 
   /* URI tags are limited to one of each per taglist */
-  if (gst_tag_list_peek_string_index (list, tag, 0, &url) && url != NULL) {
+  if (gst_tag_list_get_string_index (list, tag, 0, &url) && url != NULL) {
     guint url_len;
 
     url_len = strlen (url);
@@ -936,6 +907,8 @@ add_uri_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     } else {
       GST_WARNING ("Tag %s does not contain a valid URI (%s)", tag, url);
     }
+
+    g_free (url);
   }
 }
 
@@ -952,7 +925,7 @@ add_relative_volume_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   gint16 gain_int;
   guint8 peak_bits;
   GstId3v2Frame frame;
-  const gchar *frame_id;
+  gchar *frame_id;
 
   /* figure out tag names and the identification string to use */
   if (strcmp (tag, GST_TAG_TRACK_PEAK) == 0 ||
@@ -1060,9 +1033,6 @@ static const struct
   GST_TAG_COMMENT, add_comment_tag, NULL}, {
   GST_TAG_EXTENDED_COMMENT, add_comment_tag, NULL}, {
 
-    /* BPM tag */
-  GST_TAG_BEATS_PER_MINUTE, add_bpm_tag, NULL}, {
-
     /* Images */
   GST_TAG_IMAGE, add_image_tag, NULL}, {
   GST_TAG_PREVIEW_IMAGE, add_image_tag, NULL}, {
@@ -1183,8 +1153,7 @@ latin1_convert (const GstTagList * list, const gchar * tag,
 
   /* Convert to Latin-1 (ISO-8859-1), replacing unrepresentable characters
      with '?' */
-  latin1 =
-      g_convert_with_fallback (str, -1, "ISO-8859-1", "UTF-8", (char *) "?",
+  latin1 = g_convert_with_fallback (str, -1, "ISO-8859-1", "UTF-8", "?",
       NULL, &len, NULL);
 
   if (latin1 != NULL && *latin1 != '\0') {
@@ -1224,12 +1193,12 @@ static void
 genre_v1_convert (const GstTagList * list, const gchar * tag,
     guint8 * dst, int maxlen, gboolean * wrote_tag)
 {
-  const gchar *str;
+  gchar *str;
   int genreidx = -1;
   guint i, max;
 
   /* We only support one genre */
-  if (!gst_tag_list_peek_string_index (list, tag, 0, &str) || str == NULL)
+  if (!gst_tag_list_get_string_index (list, tag, 0, &str) || str == NULL)
     return;
 
   max = gst_tag_id3_genre_count ();
@@ -1246,6 +1215,8 @@ genre_v1_convert (const GstTagList * list, const gchar * tag,
     *dst = (guint8) genreidx;
     *wrote_tag = TRUE;
   }
+
+  g_free (str);
 }
 
 static void
