@@ -28,6 +28,12 @@
 
 #include <gst/video/video.h>
 
+static const GstElementDetails gst_dshowvideosrc_details =
+GST_ELEMENT_DETAILS ("DirectShow video capture source",
+    "Source/Video",
+    "Receive data from a directshow video capture graph",
+    "Sebastien Moutte <sebastien@moutte.net>");
+
 GST_DEBUG_CATEGORY_STATIC (dshowvideosrc_debug);
 #define GST_CAT_DEFAULT dshowvideosrc_debug
 
@@ -36,7 +42,6 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_BGR ";"
         GST_VIDEO_CAPS_YUV ("{ I420 }") ";"
-        GST_VIDEO_CAPS_YUV ("{ YUY2 }") ";"
         "video/x-dv,"
         "systemstream = (boolean) FALSE,"
         "width = (int) [ 1, MAX ],"
@@ -96,8 +101,8 @@ static GstCaps *gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc *
     src, IPin * pin);
 static GstCaps *gst_dshowvideosrc_getcaps_from_enum_mediatypes (GstDshowVideoSrc *
     src, IPin * pin);
-static gboolean gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size,
-    gpointer src_object, GstClockTime duration);
+static gboolean gst_dshowvideosrc_push_buffer (byte * buffer, long size,
+    gpointer src_object, UINT64 start, UINT64 stop);
 
 static void
 gst_dshowvideosrc_init_interfaces (GType type)
@@ -126,12 +131,10 @@ gst_dshowvideosrc_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "DirectShow video capture source", "Source/Video",
-      "Receive data from a directshow video capture graph",
-      "Sebastien Moutte <sebastien@moutte.net>");
+  gst_element_class_set_details (element_class, &gst_dshowvideosrc_details);
 }
 
 static void
@@ -1002,11 +1005,11 @@ gst_dshowvideosrc_getcaps_from_enum_mediatypes (GstDshowVideoSrc * src, IPin * p
 }
 
 static gboolean
-gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size, gpointer src_object,
-    GstClockTime duration)
+gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
+    UINT64 start, UINT64 stop)
 {
   GstDshowVideoSrc *src = GST_DSHOWVIDEOSRC (src_object);
-  GstBuffer *buf = NULL;
+  GstBuffer *buf;
   IPin *pPin = NULL;
   HRESULT hres = S_FALSE;
   AM_MEDIA_TYPE *pMediaType = NULL;
@@ -1019,13 +1022,9 @@ gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size, gpointer src_object,
   buf = gst_buffer_new_and_alloc (size);
 
   GST_BUFFER_SIZE (buf) = size;
-
-  GstClock *clock = gst_element_get_clock (GST_ELEMENT (src));
-  GST_BUFFER_TIMESTAMP (buf) =
-    GST_CLOCK_DIFF (gst_element_get_base_time (GST_ELEMENT (src)), gst_clock_get_time (clock));
-  gst_object_unref (clock);
-
-  GST_BUFFER_DURATION (buf) = duration;
+  GST_BUFFER_TIMESTAMP (buf) = gst_clock_get_time (GST_ELEMENT (src)->clock);
+  GST_BUFFER_TIMESTAMP (buf) -= GST_ELEMENT (src)->base_time;
+  GST_BUFFER_DURATION (buf) = stop - start;
 
   if (src->is_rgb) {
     /* FOR RGB directshow decoder will return bottom-up BITMAP
@@ -1045,7 +1044,7 @@ gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size, gpointer src_object,
 
   GST_DEBUG ("push_buffer => pts %" GST_TIME_FORMAT "duration %"
       GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-      GST_TIME_ARGS (duration));
+      GST_TIME_ARGS (stop - start));
 
   /* the negotiate() method already set caps on the source pad */
   gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
