@@ -25,7 +25,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch dvbsrc modulation="QAM 64" trans-mode=8k bandwidth=8 frequency=514000000 code-rate-lp=AUTO code-rate-hp=2/3 guard=4  hierarchy=0 ! mpegtsdemux name=demux ! queue max-size-buffers=0 max-size-time=0 ! mpeg2dec ! xvimagesink demux. ! queue max-size-buffers=0 max-size-time=0 ! mad ! alsasink
+ * gst-launch dvbsrc modulation="QAM 64" trans-mode=8k bandwidth=8MHz frequency=514000000 code-rate-lp=AUTO code-rate-hp=2/3 guard=4  hierarchy=0 ! mpegtsdemux name=demux ! queue max-size-buffers=0 max-size-time=0 ! mpeg2dec ! xvimagesink demux. ! queue max-size-buffers=0 max-size-time=0 ! mad ! alsasink
  * ]| Captures a full transport stream from dvb card 0 that is a DVB-T card at tuned frequency 514000000 with other parameters as seen in the pipeline and renders the first tv program on the transport stream.
  * |[
  * gst-launch dvbsrc modulation="QAM 64" trans-mode=8k bandwidth=8 frequency=514000000 code-rate-lp=AUTO code-rate-hp=2/3 guard=4  hierarchy=0 pids=100:256:257 ! mpegtsdemux name=demux ! queue max-size-buffers=0 max-size-time=0 ! mpeg2dec ! xvimagesink demux. ! queue max-size-buffers=0 max-size-time=0 ! mad ! alsasink
@@ -45,6 +45,7 @@
 #include <sys/ioctl.h>
 #include <sys/poll.h>
 #include <fcntl.h>
+#include <error.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -67,6 +68,15 @@ GST_DEBUG_CATEGORY_STATIC (gstdvbsrc_debug);
 #define LOF1 (9750*1000UL)
 #define LOF2 (10600*1000UL)
 
+
+static GstElementDetails dvbsrc_details = {
+  "DVB Source",
+  "Source/Video",
+  "Digital Video Broadcast Source",
+  "P2P-VCR, C-Lab, University of Paderborn\n"
+      "Zaheer Abbas Merali <zaheerabbas at merali dot org>"
+};
+
 /* Arguments */
 enum
 {
@@ -87,30 +97,9 @@ enum
   ARG_DVBSRC_HIERARCHY_INF,
   ARG_DVBSRC_TUNE,
   ARG_DVBSRC_INVERSION,
-  ARG_DVBSRC_STATS_REPORTING_INTERVAL,
-  ARG_DVBSRC_TIMEOUT,
+  ARG_DVBSRC_STATS_REPORTING_INTERVAL
 };
 
-#define DEFAULT_ADAPTER 0
-#define DEFAULT_FRONTEND 0
-#define DEFAULT_DISEQC_SRC -1   /* disabled */
-#define DEFAULT_FREQUENCY 0
-#define DEFAULT_POLARITY "H"
-#define DEFAULT_PIDS "8192"
-#define DEFAULT_SYMBOL_RATE 0
-#define DEFAULT_BANDWIDTH BANDWIDTH_7_MHZ
-#define DEFAULT_CODE_RATE_HP FEC_AUTO
-#define DEFAULT_CODE_RATE_LP FEC_1_2
-#define DEFAULT_GUARD GUARD_INTERVAL_1_16
-#define DEFAULT_MODULATION QAM_16
-#define DEFAULT_TRANSMISSION_MODE TRANSMISSION_MODE_8K
-#define DEFAULT_HIERARCHY HIERARCHY_1
-#define DEFAULT_INVERSION INVERSION_ON
-#define DEFAULT_STATS_REPORTING_INTERVAL 100
-#define DEFAULT_TIMEOUT 1000000 /* 1 second */
-
-#define DEFAULT_BUFFER_SIZE 8192        /* not a property */
-
 static void gst_dvbsrc_output_frontend_stats (GstDvbSrc * src);
 
 #define GST_TYPE_DVBSRC_CODE_RATE (gst_dvbsrc_code_rate_get_type ())
@@ -119,7 +108,7 @@ gst_dvbsrc_code_rate_get_type (void)
 {
   static GType dvbsrc_code_rate_type = 0;
   static GEnumValue code_rate_types[] = {
-    {FEC_NONE, "NONE", "none"},
+    {FEC_NONE, "NONE", "NONE"},
     {FEC_1_2, "1/2", "1/2"},
     {FEC_2_3, "2/3", "2/3"},
     {FEC_3_4, "3/4", "3/4"},
@@ -128,7 +117,7 @@ gst_dvbsrc_code_rate_get_type (void)
     {FEC_6_7, "6/7", "6/7"},
     {FEC_7_8, "7/8", "7/8"},
     {FEC_8_9, "8/9", "8/9"},
-    {FEC_AUTO, "AUTO", "auto"},
+    {FEC_AUTO, "AUTO", ""},
     {0, NULL, NULL},
   };
 
@@ -145,15 +134,15 @@ gst_dvbsrc_modulation_get_type (void)
 {
   static GType dvbsrc_modulation_type = 0;
   static GEnumValue modulation_types[] = {
-    {QPSK, "QPSK", "qpsk"},
-    {QAM_16, "QAM 16", "qam-16"},
-    {QAM_32, "QAM 32", "qam-32"},
-    {QAM_64, "QAM 64", "qam-64"},
-    {QAM_128, "QAM 128", "qam-128"},
-    {QAM_256, "QAM 256", "qam-256"},
-    {QAM_AUTO, "AUTO", "auto"},
-    {VSB_8, "8VSB", "8vsb"},
-    {VSB_16, "16VSB", "16vsb"},
+    {QPSK, "QPSK", "QPSK"},
+    {QAM_16, "QAM 16", "QAM 16"},
+    {QAM_32, "QAM 32", "QAM 32"},
+    {QAM_64, "QAM 64", "QAM 64"},
+    {QAM_128, "QAM 128", "QAM 128"},
+    {QAM_256, "QAM 256", "QAM 256"},
+    {QAM_AUTO, "AUTO", "AUTO"},
+    {VSB_8, "8VSB", "8VSB"},
+    {VSB_16, "16VSB", "16VSB"},
     {0, NULL, NULL},
   };
 
@@ -170,9 +159,9 @@ gst_dvbsrc_transmission_mode_get_type (void)
 {
   static GType dvbsrc_transmission_mode_type = 0;
   static GEnumValue transmission_mode_types[] = {
-    {TRANSMISSION_MODE_2K, "2K", "2k"},
-    {TRANSMISSION_MODE_8K, "8K", "8k"},
-    {TRANSMISSION_MODE_AUTO, "AUTO", "auto"},
+    {TRANSMISSION_MODE_2K, "2k", "2k"},
+    {TRANSMISSION_MODE_8K, "8k", "8k"},
+    {TRANSMISSION_MODE_AUTO, "AUTO", "AUTO"},
     {0, NULL, NULL},
   };
 
@@ -214,7 +203,7 @@ gst_dvbsrc_guard_get_type (void)
     {GUARD_INTERVAL_1_16, "16", "16"},
     {GUARD_INTERVAL_1_8, "8", "8"},
     {GUARD_INTERVAL_1_4, "4", "4"},
-    {GUARD_INTERVAL_AUTO, "AUTO", "auto"},
+    {GUARD_INTERVAL_AUTO, "AUTO", "AUTO"},
     {0, NULL, NULL},
   };
 
@@ -230,11 +219,11 @@ gst_dvbsrc_hierarchy_get_type (void)
 {
   static GType dvbsrc_hierarchy_type = 0;
   static GEnumValue hierarchy_types[] = {
-    {HIERARCHY_NONE, "NONE", "none"},
+    {HIERARCHY_NONE, "NONE", "NONE"},
     {HIERARCHY_1, "1", "1"},
     {HIERARCHY_2, "2", "2"},
     {HIERARCHY_4, "4", "4"},
-    {HIERARCHY_AUTO, "AUTO", "auto"},
+    {HIERARCHY_AUTO, "AUTO", "AUTO"},
     {0, NULL, NULL},
   };
 
@@ -251,9 +240,9 @@ gst_dvbsrc_inversion_get_type (void)
 {
   static GType dvbsrc_inversion_type = 0;
   static GEnumValue inversion_types[] = {
-    {INVERSION_OFF, "OFF", "off"},
-    {INVERSION_ON, "ON", "on"},
-    {INVERSION_AUTO, "AUTO", "auto"},
+    {INVERSION_OFF, "OFF", "OFF"},
+    {INVERSION_ON, "ON", "ON"},
+    {INVERSION_AUTO, "AUTO", "AUTO"},
     {0, NULL, NULL},
   };
 
@@ -279,7 +268,6 @@ static GstStateChangeReturn gst_dvbsrc_change_state (GstElement * element,
     GstStateChange transition);
 
 static gboolean gst_dvbsrc_unlock (GstBaseSrc * bsrc);
-static gboolean gst_dvbsrc_unlock_stop (GstBaseSrc * bsrc);
 static gboolean gst_dvbsrc_is_seekable (GstBaseSrc * bsrc);
 static gboolean gst_dvbsrc_get_size (GstBaseSrc * src, guint64 * size);
 
@@ -313,14 +301,10 @@ gst_dvbsrc_base_init (gpointer gclass)
   GstDvbSrcClass *klass = (GstDvbSrcClass *) gclass;
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &ts_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ts_src_factory));
 
-  gst_element_class_set_details_simple (element_class, "DVB Source",
-      "Source/Video",
-      "Digital Video Broadcast Source",
-      "P2P-VCR, C-Lab, University of Paderborn,"
-      "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
+  gst_element_class_set_details (element_class, &dvbsrc_details);
 }
 
 
@@ -346,40 +330,42 @@ gst_dvbsrc_class_init (GstDvbSrcClass * klass)
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_dvbsrc_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_dvbsrc_stop);
   gstbasesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_dvbsrc_unlock);
-  gstbasesrc_class->unlock_stop = GST_DEBUG_FUNCPTR (gst_dvbsrc_unlock_stop);
   gstbasesrc_class->is_seekable = GST_DEBUG_FUNCPTR (gst_dvbsrc_is_seekable);
   gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (gst_dvbsrc_get_size);
 
-  gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_dvbsrc_create);
-
+  gstpushsrc_class->create = gst_dvbsrc_create;
   g_object_class_install_property (gobject_class, ARG_DVBSRC_ADAPTER,
-      g_param_spec_int ("adapter", "The adapter device number",
-          "The adapter device number (eg. 0 for adapter0)",
-          0, 16, DEFAULT_ADAPTER, G_PARAM_READWRITE));
+      g_param_spec_int ("adapter",
+          "The adapter device number",
+          "The adapter device number (eg. 0 for adapter0)", 0, 16, 0,
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_FRONTEND,
-      g_param_spec_int ("frontend", "The frontend device number",
-          "The frontend device number (eg. 0 for frontend0)",
-          0, 16, DEFAULT_FRONTEND, G_PARAM_READWRITE));
+      g_param_spec_int ("frontend",
+          "The frontend device number",
+          "The frontend device number (eg. 0 for frontend0)", 0, 16, 0,
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_FREQUENCY,
-      g_param_spec_uint ("frequency", "frequency", "Frequency",
-          0, G_MAXUINT, DEFAULT_FREQUENCY, G_PARAM_READWRITE));
+      g_param_spec_int ("frequency",
+          "frequency", "Frequency", 0, G_MAXINT, 0, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_POLARITY,
-      g_param_spec_string ("polarity", "polarity", "Polarity [vhHV] (DVB-S)",
-          DEFAULT_POLARITY, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+      g_param_spec_string ("polarity",
+          "polarity", "Polarity [vhHV] (DVB-S)", "h", G_PARAM_READWRITE));
+
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_PIDS,
-      g_param_spec_string ("pids", "pids",
+      g_param_spec_string ("pids",
+          "pids",
           "Colon seperated list of pids (eg. 110:120)",
-          DEFAULT_PIDS, G_PARAM_WRITABLE));
+          "8192", G_PARAM_WRITABLE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_SYM_RATE,
-      g_param_spec_uint ("symbol-rate",
+      g_param_spec_int ("symbol-rate",
           "symbol rate",
           "Symbol Rate (DVB-S, DVB-C)",
-          0, G_MAXUINT, DEFAULT_SYMBOL_RATE, G_PARAM_READWRITE));
+          0, G_MAXINT, DEFAULT_SYMBOL_RATE, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_TUNE,
       g_param_spec_pointer ("tune",
@@ -396,61 +382,57 @@ gst_dvbsrc_class_init (GstDvbSrcClass * klass)
   g_object_class_install_property (gobject_class, ARG_DVBSRC_BANDWIDTH,
       g_param_spec_enum ("bandwidth",
           "bandwidth",
-          "Bandwidth (DVB-T)", GST_TYPE_DVBSRC_BANDWIDTH, DEFAULT_BANDWIDTH,
+          "Bandwidth (DVB-T)", GST_TYPE_DVBSRC_BANDWIDTH, 1,
           G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_CODE_RATE_HP,
       g_param_spec_enum ("code-rate-hp",
           "code-rate-hp",
           "High Priority Code Rate (DVB-T, DVB-S and DVB-C)",
-          GST_TYPE_DVBSRC_CODE_RATE, DEFAULT_CODE_RATE_HP, G_PARAM_READWRITE));
+          GST_TYPE_DVBSRC_CODE_RATE, FEC_AUTO, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_CODE_RATE_LP,
       g_param_spec_enum ("code-rate-lp",
           "code-rate-lp",
           "Low Priority Code Rate (DVB-T)",
-          GST_TYPE_DVBSRC_CODE_RATE, DEFAULT_CODE_RATE_LP, G_PARAM_READWRITE));
+          GST_TYPE_DVBSRC_CODE_RATE, 1, G_PARAM_READWRITE));
 
-  /* FIXME: should the property be called 'guard-interval' then? */
   g_object_class_install_property (gobject_class, ARG_DVBSRC_GUARD,
       g_param_spec_enum ("guard",
           "guard",
           "Guard Interval (DVB-T)",
-          GST_TYPE_DVBSRC_GUARD, DEFAULT_GUARD, G_PARAM_READWRITE));
+          GST_TYPE_DVBSRC_GUARD, 1, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_MODULATION,
-      g_param_spec_enum ("modulation", "modulation",
+      g_param_spec_enum ("modulation",
+          "modulation",
           "Modulation (DVB-T and DVB-C)",
-          GST_TYPE_DVBSRC_MODULATION, DEFAULT_MODULATION, G_PARAM_READWRITE));
+          GST_TYPE_DVBSRC_MODULATION, 1, G_PARAM_READWRITE));
 
-  /* FIXME: property should be named 'transmission-mode' */
   g_object_class_install_property (gobject_class,
       ARG_DVBSRC_TRANSMISSION_MODE,
-      g_param_spec_enum ("trans-mode", "trans-mode",
-          "Transmission Mode (DVB-T)", GST_TYPE_DVBSRC_TRANSMISSION_MODE,
-          DEFAULT_TRANSMISSION_MODE, G_PARAM_READWRITE));
+      g_param_spec_enum ("trans-mode",
+          "trans-mode",
+          "Transmission Mode (DVB-T)",
+          GST_TYPE_DVBSRC_TRANSMISSION_MODE, 1, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_DVBSRC_HIERARCHY_INF,
-      g_param_spec_enum ("hierarchy", "hierarchy",
+      g_param_spec_enum ("hierarchy",
+          "hierarchy",
           "Hierarchy Information (DVB-T)",
-          GST_TYPE_DVBSRC_HIERARCHY, DEFAULT_HIERARCHY, G_PARAM_READWRITE));
-
+          GST_TYPE_DVBSRC_HIERARCHY, 1, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_DVBSRC_INVERSION,
-      g_param_spec_enum ("inversion", "inversion",
+      g_param_spec_enum ("inversion",
+          "inversion",
           "Inversion Information (DVB-T and DVB-C)",
-          GST_TYPE_DVBSRC_INVERSION, DEFAULT_INVERSION, G_PARAM_READWRITE));
+          GST_TYPE_DVBSRC_INVERSION, 1, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
       ARG_DVBSRC_STATS_REPORTING_INTERVAL,
       g_param_spec_uint ("stats-reporting-interval",
           "stats-reporting-interval",
           "The number of reads before reporting frontend stats",
-          0, G_MAXUINT, DEFAULT_STATS_REPORTING_INTERVAL, G_PARAM_READWRITE));
-
-  g_object_class_install_property (gobject_class, ARG_DVBSRC_TIMEOUT,
-      g_param_spec_uint64 ("timeout", "Timeout",
-          "Post a message after timeout microseconds (0 = disabled)", 0,
-          G_MAXUINT64, DEFAULT_TIMEOUT, G_PARAM_READWRITE));
+          0, G_MAXUINT, 100, G_PARAM_READWRITE));
 }
 
 /* initialize the new element
@@ -478,24 +460,13 @@ gst_dvbsrc_init (GstDvbSrc * object, GstDvbSrcClass * klass)
   /* Pid 8192 on DVB gets the whole transport stream */
   object->pids[0] = 8192;
 
-  object->adapter_number = DEFAULT_ADAPTER;
-  object->frontend_number = DEFAULT_FRONTEND;
-  object->diseqc_src = DEFAULT_DISEQC_SRC;
-  object->send_diseqc = (DEFAULT_DISEQC_SRC != -1);
-  /* object->pol = DVB_POL_H; *//* set via G_PARAM_CONSTRUCT */
+  object->adapter_number = 0;
+  object->frontend_number = 0;
   object->sym_rate = DEFAULT_SYMBOL_RATE;
-  object->bandwidth = DEFAULT_BANDWIDTH;
-  object->code_rate_hp = DEFAULT_CODE_RATE_HP;
-  object->code_rate_lp = DEFAULT_CODE_RATE_LP;
-  object->guard_interval = DEFAULT_GUARD;
-  object->modulation = DEFAULT_MODULATION;
-  object->transmission_mode = DEFAULT_TRANSMISSION_MODE;
-  object->hierarchy_information = DEFAULT_HIERARCHY;
-  object->inversion = DEFAULT_INVERSION;
-  object->stats_interval = DEFAULT_STATS_REPORTING_INTERVAL;
-
+  object->diseqc_src = DEFAULT_DISEQC_SRC;
+  object->send_diseqc = FALSE;
+  object->code_rate_hp = FEC_AUTO;
   object->tune_mutex = g_mutex_new ();
-  object->timeout = DEFAULT_TIMEOUT;
 }
 
 
@@ -523,7 +494,7 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
       GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_DISEQC_ID");
       break;
     case ARG_DVBSRC_FREQUENCY:
-      object->freq = g_value_get_uint (value);
+      object->freq = g_value_get_int (value);
       GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_FREQUENCY");
       break;
     case ARG_DVBSRC_POLARITY:
@@ -589,7 +560,7 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
     }
       break;
     case ARG_DVBSRC_SYM_RATE:
-      object->sym_rate = g_value_get_uint (value);
+      object->sym_rate = g_value_get_int (value);
       GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_SYM_RATE to value %d",
           g_value_get_int (value));
       break;
@@ -618,27 +589,25 @@ gst_dvbsrc_set_property (GObject * _object, guint prop_id,
     case ARG_DVBSRC_INVERSION:
       object->inversion = g_value_get_enum (value);
       break;
-    case ARG_DVBSRC_TUNE:{
+    case ARG_DVBSRC_TUNE:
       GST_INFO_OBJECT (object, "Set Property: ARG_DVBSRC_TUNE");
-
       /* if we are in paused/playing state tune now, otherwise in ready to paused state change */
-      if (GST_STATE (object) > GST_STATE_READY) {
+      if (gst_element_get_state
+          (GST_ELEMENT (object), NULL, NULL,
+              GST_CLOCK_TIME_NONE) > GST_STATE_READY) {
         g_mutex_lock (object->tune_mutex);
         gst_dvbsrc_tune (object);
         g_mutex_unlock (object->tune_mutex);
       }
       break;
-    }
     case ARG_DVBSRC_STATS_REPORTING_INTERVAL:
       object->stats_interval = g_value_get_uint (value);
       object->stats_counter = 0;
       break;
-    case ARG_DVBSRC_TIMEOUT:
-      object->timeout = g_value_get_uint64 (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
+
 }
 
 static void
@@ -658,16 +627,16 @@ gst_dvbsrc_get_property (GObject * _object, guint prop_id,
       g_value_set_int (value, object->frontend_number);
       break;
     case ARG_DVBSRC_FREQUENCY:
-      g_value_set_uint (value, object->freq);
+      g_value_set_int (value, object->freq);
       break;
     case ARG_DVBSRC_POLARITY:
       if (object->pol == DVB_POL_H)
-        g_value_set_static_string (value, "H");
+        g_value_set_string (value, "H");
       else
-        g_value_set_static_string (value, "V");
+        g_value_set_string (value, "V");
       break;
     case ARG_DVBSRC_SYM_RATE:
-      g_value_set_uint (value, object->sym_rate);
+      g_value_set_int (value, object->sym_rate);
       break;
     case ARG_DVBSRC_DISEQC_SRC:
       g_value_set_int (value, object->diseqc_src);
@@ -699,9 +668,6 @@ gst_dvbsrc_get_property (GObject * _object, guint prop_id,
     case ARG_DVBSRC_STATS_REPORTING_INTERVAL:
       g_value_set_uint (value, object->stats_interval);
       break;
-    case ARG_DVBSRC_TIMEOUT:
-      g_value_set_uint64 (value, object->timeout);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -724,7 +690,7 @@ static gboolean
 gst_dvbsrc_open_frontend (GstDvbSrc * object)
 {
   struct dvb_frontend_info fe_info;
-  const char *adapter_desc = NULL;
+  char *adapter_desc = NULL;
   gchar *frontend_dev;
   GstStructure *adapter_structure;
   char *adapter_name = NULL;
@@ -901,62 +867,75 @@ gst_dvbsrc_plugin_init (GstPlugin * plugin)
 }
 
 static GstBuffer *
-gst_dvbsrc_read_device (GstDvbSrc * object, int size)
+read_device (int fd, int adapter_number, int frontend_number, int size,
+    GstDvbSrc * object)
 {
-  gint count = 0;
-  gint ret_val = 0;
+  int count = 0;
+  struct pollfd pfd[1];
+  int ret_val = 0;
+  guint attempts = 0;
+  const int TIMEOUT = 100;
+
   GstBuffer *buf = gst_buffer_new_and_alloc (size);
-  GstClockTime timeout = object->timeout * GST_USECOND;
 
   g_return_val_if_fail (GST_IS_BUFFER (buf), NULL);
 
-  if (object->fd_dvr < 0)
+  if (fd < 0) {
     return NULL;
+  }
+
+  pfd[0].fd = fd;
+  pfd[0].events = POLLIN;
 
   while (count < size) {
-    ret_val = gst_poll_wait (object->poll, timeout);
-    GST_LOG_OBJECT (object, "select returned %d", ret_val);
-    if (G_UNLIKELY (ret_val < 0)) {
-      if (errno == EBUSY)
-        goto stopped;
-      else
-        goto select_error;
-    } else if (G_UNLIKELY (ret_val == 0)) {
-      /* timeout, post element message */
-      gst_element_post_message (GST_ELEMENT_CAST (object),
-          gst_message_new_element (GST_OBJECT (object),
-              gst_structure_empty_new ("dvb-read-failure")));
-    } else {
-      int nread =
-          read (object->fd_dvr, GST_BUFFER_DATA (buf) + count, size - count);
+    ret_val = poll (pfd, 1, TIMEOUT);
+    if (ret_val > 0) {
+      if (pfd[0].revents & POLLIN) {
+        int tmp = 0;
 
-      if (G_UNLIKELY (nread < 0)) {
-        GST_WARNING_OBJECT
-            (object,
-            "Unable to read from device: /dev/dvb/adapter%d/dvr%d (%d)",
-            object->adapter_number, object->frontend_number, errno);
+        tmp = read (fd, GST_BUFFER_DATA (buf) + count, size - count);
+        if (tmp < 0) {
+          GST_WARNING
+              ("Unable to read from device: /dev/dvb/adapter%d/dvr%d (%d)",
+              adapter_number, frontend_number, errno);
+          attempts += 1;
+          if (attempts % 10 == 0) {
+            GST_WARNING
+                ("Unable to read from device after %u attempts: /dev/dvb/adapter%d/dvr%d",
+                attempts, adapter_number, frontend_number);
+          }
+
+        } else
+          count = count + tmp;
+      } else {
+        GST_LOG ("revents = %d\n", pfd[0].revents);
+      }
+    } else if (ret_val == 0) {  // poll timeout
+      attempts += 1;
+      GST_INFO ("Reading from device /dev/dvb/adapter%d/dvr%d timedout (%d)",
+          adapter_number, frontend_number, attempts);
+
+      if (attempts % 10 == 0) {
+        GST_WARNING
+            ("Unable to read after %u attempts from device: /dev/dvb/adapter%d/dvr%d (%d)",
+            attempts, adapter_number, frontend_number, errno);
         gst_element_post_message (GST_ELEMENT_CAST (object),
             gst_message_new_element (GST_OBJECT (object),
                 gst_structure_empty_new ("dvb-read-failure")));
-      } else
-        count = count + nread;
+
+      }
+    } else if (errno == -EINTR) {       // poll interrupted
+      if (attempts % 50 == 0) {
+        gst_buffer_unref (buf);
+        return NULL;
+      };
     }
+
   }
 
   GST_BUFFER_SIZE (buf) = count;
   GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
   return buf;
-
-stopped:
-  GST_DEBUG_OBJECT (object, "stop called");
-  gst_buffer_unref (buf);
-  return NULL;
-
-select_error:
-  GST_ELEMENT_ERROR (object, RESOURCE, READ, (NULL),
-      ("select error %d: %s (%d)", ret_val, g_strerror (errno), errno));
-  gst_buffer_unref (buf);
-  return NULL;
 }
 
 static GstFlowReturn
@@ -979,7 +958,8 @@ gst_dvbsrc_create (GstPushSrc * element, GstBuffer ** buf)
   if (object->fd_dvr > -1) {
     /* --- Read TS from DVR device --- */
     GST_DEBUG_OBJECT (object, "Reading from DVR device");
-    *buf = gst_dvbsrc_read_device (object, buffer_size);
+    *buf = read_device (object->fd_dvr, object->adapter_number,
+        object->frontend_number, buffer_size, object);
     if (*buf != NULL) {
       GstCaps *caps;
 
@@ -988,6 +968,11 @@ gst_dvbsrc_create (GstPushSrc * element, GstBuffer ** buf)
       caps = gst_pad_get_caps (GST_BASE_SRC_PAD (object));
       gst_buffer_set_caps (*buf, caps);
       gst_caps_unref (caps);
+    } else {
+      GST_DEBUG_OBJECT (object, "Failed to read from device");
+      gst_element_post_message (GST_ELEMENT_CAST (object),
+          gst_message_new_element (GST_OBJECT (object),
+              gst_structure_empty_new ("dvb-read-failure")));
     }
 
     if (object->stats_interval != 0 &&
@@ -1051,19 +1036,6 @@ gst_dvbsrc_start (GstBaseSrc * bsrc)
     close (src->fd_frontend);
     return FALSE;
   }
-  if (!(src->poll = gst_poll_new (TRUE))) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ_WRITE, (NULL),
-        ("could not create an fdset: %s (%d)", g_strerror (errno), errno));
-    /* unset filters also */
-    gst_dvbsrc_unset_pes_filters (src);
-    close (src->fd_frontend);
-    return FALSE;
-  } else {
-    gst_poll_fd_init (&src->poll_fd_dvr);
-    src->poll_fd_dvr.fd = src->fd_dvr;
-    gst_poll_add_fd (src->poll, &src->poll_fd_dvr);
-    gst_poll_fd_ctl_read (src->poll, &src->poll_fd_dvr, TRUE);
-  }
 
   return TRUE;
 }
@@ -1074,29 +1046,12 @@ gst_dvbsrc_stop (GstBaseSrc * bsrc)
   GstDvbSrc *src = GST_DVBSRC (bsrc);
 
   gst_dvbsrc_close_devices (src);
-  if (src->poll) {
-    gst_poll_free (src->poll);
-    src->poll = NULL;
-  }
-
   return TRUE;
 }
 
 static gboolean
 gst_dvbsrc_unlock (GstBaseSrc * bsrc)
 {
-  GstDvbSrc *src = GST_DVBSRC (bsrc);
-
-  gst_poll_set_flushing (src->poll, TRUE);
-  return TRUE;
-}
-
-static gboolean
-gst_dvbsrc_unlock_stop (GstBaseSrc * bsrc)
-{
-  GstDvbSrc *src = GST_DVBSRC (bsrc);
-
-  gst_poll_set_flushing (src->poll, FALSE);
   return TRUE;
 }
 
@@ -1332,9 +1287,6 @@ gst_dvbsrc_tune (GstDvbSrc * object)
 
         feparams.frequency = freq;
 #if DVB_API_VERSION == 3 && DVB_API_VERSION_MINOR == 3
-        feparams.delsys.dvbs.fec = object->code_rate_hp;
-        feparams.delsys.dvbs.modulation = object->modulation;
-        feparams.delsys.dvbs.symbol_rate = sym_rate;
 #else
         feparams.u.ofdm.bandwidth = object->bandwidth;
         feparams.u.ofdm.code_rate_HP = object->code_rate_hp;
@@ -1353,24 +1305,14 @@ gst_dvbsrc_tune (GstDvbSrc * object)
             sym_rate);
         feparams.frequency = freq;
         feparams.inversion = object->inversion;
-#if DVB_API_VERSION == 3 && DVB_API_VERSION_MINOR == 3
-        feparams.delsys.dvbs.fec = object->code_rate_hp;
-        feparams.delsys.dvbs.modulation = object->modulation;
-        feparams.delsys.dvbs.symbol_rate = sym_rate;
-#else
         feparams.u.qam.fec_inner = object->code_rate_hp;
         feparams.u.qam.modulation = object->modulation;
         feparams.u.qam.symbol_rate = sym_rate;
-#endif
         break;
       case FE_ATSC:
         GST_INFO_OBJECT (object, "Tuning ATSC to %d", freq);
         feparams.frequency = freq;
-#if DVB_API_VERSION == 3 && DVB_API_VERSION_MINOR == 3
-        feparams.delsys.atsc.modulation = object->modulation;
-#else
         feparams.u.vsb.modulation = object->modulation;
-#endif
         break;
       default:
         g_error ("Unknown frontend type: %d", object->adapter_type);
