@@ -44,7 +44,7 @@
  * Includes
  */
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include "config.h"
 #endif
 
 /* save the snapshot images
@@ -197,7 +197,7 @@ pad_has_buffer (GstPad * pad, GstBuffer * buf, gpointer user_data)
       case 8:                  // video recording start
         DIFF_TIME (t_final[num_pics_cont], t_initial, diff);
         result.avg = result.min = result.max = diff;
-        //g_signal_emit_by_name (camera_bin, "capture-stop", 0);
+        //g_signal_emit_by_name (camera_bin, "user-stop", 0);
         print_and_restart = TRUE;
         break;
       default:
@@ -223,7 +223,7 @@ element_added (GstBin * bin, GstElement * element, gpointer user_data)
   }
 
   if (need_vmux_pad_probe) {
-    g_object_get (camera_bin, "video-muxer", &elem, NULL);
+    g_object_get (camera_bin, "videomux", &elem, NULL);
     if (elem) {
       need_vmux_pad_probe = FALSE;
       GST_INFO_OBJECT (elem, "got default video muxer");
@@ -235,7 +235,7 @@ element_added (GstBin * bin, GstElement * element, gpointer user_data)
     }
   }
   if (need_ienc_pad_probe) {
-    g_object_get (camera_bin, "image-encoder", &elem, NULL);
+    g_object_get (camera_bin, "imageenc", &elem, NULL);
     if (elem) {
       need_ienc_pad_probe = FALSE;
       GST_INFO_OBJECT (elem, "got default image encoder");
@@ -291,7 +291,7 @@ img_capture_done (GstElement * camera, GString * fname, gpointer user_data)
       // but this needs sync so that we have received "image-captured" message already
       if (have_img_captured) {
         have_img_captured = FALSE;
-        g_signal_emit_by_name (camera_bin, "capture-start", NULL);
+        g_signal_emit_by_name (camera_bin, "user-start", NULL);
       } else {
         have_img_done = TRUE;
       }
@@ -300,6 +300,8 @@ img_capture_done (GstElement * camera, GString * fname, gpointer user_data)
       GstClockTime max = 0;
       GstClockTime min = -1;
       GstClockTime total = 0;
+      GstClockTime first_shot = 0;
+      GstClockTime snd_shot = 0;
 
       num_pics_cont = 0;
       signal_cont = FALSE;
@@ -307,11 +309,13 @@ img_capture_done (GstElement * camera, GString * fname, gpointer user_data)
       DIFF_TIME (t_final[0], t_initial, diff);
       max < diff ? max = diff : max;
       min > diff ? min = diff : min;
+      first_shot = diff;
       total += diff;
 
       DIFF_TIME (t_final[1], t_final[0], diff);
       max < diff ? max = diff : max;
       min > diff ? min = diff : min;
+      snd_shot = diff;
       total += diff;
 
       for (i = 2; i < CONT_SHOTS; ++i) {
@@ -412,7 +416,7 @@ bus_callback (GstBus * bus, GstMessage * message, gpointer data)
               // we need to have this received before we can take next one
               if (have_img_done) {
                 have_img_done = FALSE;
-                g_signal_emit_by_name (camera_bin, "capture-start", NULL);
+                g_signal_emit_by_name (camera_bin, "user-start", NULL);
               } else {
                 have_img_captured = TRUE;
               }
@@ -534,8 +538,7 @@ setup_pipeline (void)
     goto error;
   }
 
-  g_signal_connect (camera_bin, "image-done", (GCallback) img_capture_done,
-      NULL);
+  g_signal_connect (camera_bin, "img-done", (GCallback) img_capture_done, NULL);
 
   bus = gst_pipeline_get_bus (GST_PIPELINE (camera_bin));
   gst_bus_add_watch (bus, bus_callback, NULL);
@@ -544,26 +547,19 @@ setup_pipeline (void)
   GST_INFO_OBJECT (camera_bin, "camerabin created");
 
   /* configure used elements */
-  res &= setup_pipeline_element ("viewfinder-sink", "fakesink", &sink);
-  res &= setup_pipeline_element ("audio-source", audiosrc_name, NULL);
-  res &= setup_pipeline_element ("video-source", videosrc_name, NULL);
-  res &= setup_pipeline_element ("audio-encoder", audioenc_name, NULL);
-  res &= setup_pipeline_element ("video-encoder", videoenc_name, NULL);
-  res &= setup_pipeline_element ("image-encoder", imageenc_name, &ienc);
-  res &= setup_pipeline_element ("video-muxer", videomux_name, &vmux);
+  res &= setup_pipeline_element ("vfsink", "fakesink", &sink);
+  res &= setup_pipeline_element ("audiosrc", audiosrc_name, NULL);
+  res &= setup_pipeline_element ("videosrc", videosrc_name, NULL);
+  res &= setup_pipeline_element ("audioenc", audioenc_name, NULL);
+  res &= setup_pipeline_element ("videoenc", videoenc_name, NULL);
+  res &= setup_pipeline_element ("imageenc", imageenc_name, &ienc);
+  res &= setup_pipeline_element ("videomux", videomux_name, &vmux);
   if (!res) {
     goto error;
   }
 
   GST_INFO_OBJECT (camera_bin, "elements created");
 
-  if (GST_STATE_CHANGE_FAILURE ==
-      gst_element_set_state (camera_bin, GST_STATE_READY)) {
-    g_warning ("can't set camerabin to ready\n");
-    goto error;
-  }
-  GST_INFO_OBJECT (camera_bin, "camera ready");
-
   /* set properties */
   g_object_set (camera_bin, "filename", filename->str, NULL);
 
@@ -594,7 +590,7 @@ setup_pipeline (void)
     goto error;
   }
   if (!vmux) {
-    g_object_get (camera_bin, "video-muxer", &vmux, NULL);
+    g_object_get (camera_bin, "videomux", &vmux, NULL);
     if (!vmux) {
       need_pad_probe = need_vmux_pad_probe = TRUE;
       /* only run the test if we later get the element */
@@ -609,7 +605,7 @@ setup_pipeline (void)
     }
   }
   if (!ienc) {
-    g_object_get (camera_bin, "image-encoder", &ienc, NULL);
+    g_object_get (camera_bin, "imageenc", &ienc, NULL);
     if (!ienc) {
       need_pad_probe = need_ienc_pad_probe = TRUE;
       /* only run the test if we later get the element */
@@ -631,15 +627,22 @@ setup_pipeline (void)
 
   /* configure a resolution and framerate for video and viewfinder */
   if (image_width && image_height) {
-    g_signal_emit_by_name (camera_bin, "set-image-resolution", image_width,
+    g_signal_emit_by_name (camera_bin, "user-image-res", image_width,
         image_height, NULL);
   }
   /* configure a resolution and framerate for video and viewfinder */
   if (video_width && video_height && view_framerate_num && view_framerate_den) {
-    g_signal_emit_by_name (camera_bin, "set-video-resolution-fps", video_width,
+    g_signal_emit_by_name (camera_bin, "user-res-fps", video_width,
         video_height, view_framerate_num, view_framerate_den, NULL);
   }
 
+  if (GST_STATE_CHANGE_FAILURE ==
+      gst_element_set_state (camera_bin, GST_STATE_READY)) {
+    g_warning ("can't set camerabin to ready\n");
+    goto error;
+  }
+  GST_INFO_OBJECT (camera_bin, "camera ready");
+
   if (GST_STATE_CHANGE_FAILURE ==
       gst_element_set_state (camera_bin, GST_STATE_PLAYING)) {
     g_warning ("can't set camerabin to playing\n");
@@ -703,7 +706,7 @@ test_03 (void)
   g_object_set (camera_bin, "mode", 0, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "capture-start", 0);
+  g_signal_emit_by_name (camera_bin, "user-start", 0);
 
   /* the actual results are fetched in bus_callback::preview-image */
   result.times = 1;
@@ -720,7 +723,7 @@ test_04 (void)
   /* switch to image mode */
   g_object_set (camera_bin, "mode", 0, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "capture-start", 0);
+  g_signal_emit_by_name (camera_bin, "user-start", 0);
 
   /* the actual results are fetched in bus_callback::image-captured */
   result.times = 1;
@@ -740,7 +743,7 @@ test_05 (void)
   /* switch to image mode */
   g_object_set (camera_bin, "mode", 0, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "capture-start", 0);
+  g_signal_emit_by_name (camera_bin, "user-start", 0);
 
   /* the actual results are fetched in img_capture_done */
   result.times = CONT_SHOTS;
@@ -761,7 +764,7 @@ test_06 (void)
   g_object_set (camera_bin, "mode", 0, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "capture-start", 0);
+  g_signal_emit_by_name (camera_bin, "user-start", 0);
 
   /* the actual results are fetched in pad_has_buffer */
   result.times = 1;
@@ -783,7 +786,7 @@ test_07 (void)
   g_object_set (camera_bin, "mode", 0, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "capture-start", 0);
+  g_signal_emit_by_name (camera_bin, "user-start", 0);
   /* the actual results are fetched in img_capture_done */
   result.times = 1;
   return FALSE;
@@ -848,7 +851,7 @@ test_09 (void)
   g_object_set (camera_bin, "mode", 1, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "capture-start", 0);
+  g_signal_emit_by_name (camera_bin, "user-start", 0);
 
   /* the actual results are fetched in pad_has_buffer */
   result.times = 1;
